@@ -155,6 +155,7 @@ constexpr EVENT_DESCRIPTOR ThreadPoolWorkerThreadAdjustmentAdjustment = { 4, 0x1
 constexpr EVENT_DESCRIPTOR ThreadPoolWorkerThreadAdjustmentStats = { 5, 0x10000 };
 constexpr EVENT_DESCRIPTOR ThreadPoolWorkerThreadWait = { 4, 0x10000 };
 constexpr EVENT_DESCRIPTOR YieldProcessorMeasurement = { 4, 0x10000 };
+constexpr EVENT_DESCRIPTOR ThreadPoolMinMaxThreads = { 4, 0x10000 };
 constexpr EVENT_DESCRIPTOR ThreadPoolWorkingThreadCount = { 5, 0x10000 };
 constexpr EVENT_DESCRIPTOR ThreadPoolEnqueue = { 5, 0x80010000 };
 constexpr EVENT_DESCRIPTOR ThreadPoolDequeue = { 5, 0x80010000 };
@@ -215,6 +216,7 @@ constexpr EVENT_DESCRIPTOR MethodJitInliningFailedAnsi = { 5, 0x1000 };
 constexpr EVENT_DESCRIPTOR MethodJitTailCallSucceeded = { 5, 0x1000 };
 constexpr EVENT_DESCRIPTOR MethodJitTailCallFailedAnsi = { 5, 0x1000 };
 constexpr EVENT_DESCRIPTOR MethodILToNativeMap = { 5, 0x20000 };
+constexpr EVENT_DESCRIPTOR MethodILToNativeMap_V1 = { 5, 0x20000 };
 constexpr EVENT_DESCRIPTOR MethodJitTailCallFailed = { 5, 0x1000 };
 constexpr EVENT_DESCRIPTOR MethodJitInliningFailed = { 5, 0x1000 };
 constexpr EVENT_DESCRIPTOR ModuleDCStartV2 = { 4, 0x8 };
@@ -318,7 +320,9 @@ constexpr EVENT_DESCRIPTOR DCStartInit_V1 = { 4, 0x20038 };
 constexpr EVENT_DESCRIPTOR DCEndInit = { 4, 0x20038 };
 constexpr EVENT_DESCRIPTOR DCEndInit_V1 = { 4, 0x20038 };
 constexpr EVENT_DESCRIPTOR MethodDCStartILToNativeMap = { 5, 0x20000 };
+constexpr EVENT_DESCRIPTOR MethodDCStartILToNativeMap_V1 = { 5, 0x20000 };
 constexpr EVENT_DESCRIPTOR MethodDCEndILToNativeMap = { 5, 0x20000 };
+constexpr EVENT_DESCRIPTOR MethodDCEndILToNativeMap_V1 = { 5, 0x20000 };
 constexpr EVENT_DESCRIPTOR DomainModuleDCStart = { 4, 0x8 };
 constexpr EVENT_DESCRIPTOR DomainModuleDCStart_V1 = { 4, 0x8 };
 constexpr EVENT_DESCRIPTOR DomainModuleDCEnd = { 4, 0x8 };
@@ -362,9 +366,10 @@ __attribute__((weak)) LTTNG_TRACE_CONTEXT MICROSOFT_WINDOWS_DOTNETRUNTIME_PRIVAT
 #define CLR_INTEROP_KEYWORD 0x2000
 #define CLR_PRIVATEGCHANDLE_KEYWORD 0x4000
 #define CLR_PRIVATEMULTICOREJIT_KEYWORD 0x20000
+#define CLR_JITTED_METHOD_RICH_DEBUG_INFO_KEYWORD 0x40000
+#define CLR_PERFTRACK_PRIVATE_KEYWORD 0x20000000
 #define CLR_PRIVATESTACK_KEYWORD 0x40000000
 #define CLR_PRIVATESTARTUP_KEYWORD 0x80000000
-#define CLR_PERFTRACK_PRIVATE_KEYWORD 0x20000000
 #define CLR_PRIVATE_DYNAMICTYPEUSAGE_KEYWORD 0x00000020
 constexpr EVENT_DESCRIPTOR GCDecision = { 4, 0x1 };
 constexpr EVENT_DESCRIPTOR GCDecision_V1 = { 4, 0x1 };
@@ -524,6 +529,7 @@ constexpr EVENT_DESCRIPTOR FieldTransparencyComputationEnd = { 4, 0x400 };
 constexpr EVENT_DESCRIPTOR TokenTransparencyComputationStart = { 4, 0x400 };
 constexpr EVENT_DESCRIPTOR TokenTransparencyComputationEnd = { 4, 0x400 };
 constexpr EVENT_DESCRIPTOR NgenBindEvent = { 4, 0x8 };
+constexpr EVENT_DESCRIPTOR JittedMethodRichDebugInfo = { 4, 0x40000 };
 constexpr EVENT_DESCRIPTOR FailFast = { 1, 0x0 };
 constexpr EVENT_DESCRIPTOR PrvFinalizeObject = { 5, 0x1 };
 constexpr EVENT_DESCRIPTOR CCWRefCountChangeAnsi = { 5, 0x2000 };
