@@ -126,7 +126,6 @@ struct MSLAYOUT DebuggerIPCRuntimeOffsets
     void   *m_signalHijackCompleteBPAddr;
     void   *m_excepNotForRuntimeBPAddr;
     void   *m_notifyRSOfSyncCompleteBPAddr;
-    void   *m_raiseExceptionAddr;                       // The address of kernel32!RaiseException in the debuggee
     DWORD   m_debuggerWordTLSIndex;                     // The TLS slot for the debugger word used in the debugger hijack functions
 #endif // FEATURE_INTEROP_DEBUGGING
     SIZE_T  m_TLSIndex;                                 // The TLS index of the thread-local storage for coreclr.dll
@@ -154,6 +153,7 @@ struct MSLAYOUT DebuggerIPCRuntimeOffsets
     SIZE_T  m_cbOpcode;                                 // Max size of opcode
     SIZE_T  m_offTraceType;                             // Offset of the trace.type within a patch
     DWORD   m_traceTypeUnmanaged;                       // TRACE_UNMANAGED
+    void   *m_setThreadContextNeededAddr;               // Address of SetThreadContextNeededFlare
 
     DebuggerIPCRuntimeOffsets()
     {
@@ -364,30 +364,6 @@ struct MSLAYOUT DebuggerIPCControlBlockTransport
 #include "dbgtransportsession.h"
 #endif // defined(FEATURE_DBGIPC_TRANSPORT_VM) || defined(FEATURE_DBGIPC_TRANSPORT_DI)
 
-#if defined(TARGET_X86) && !defined(FEATURE_CORESYSTEM)
-// We have an versioning requirement.
-// Certain portions of the v1.0 and v1.1 IPC block are shared. This is b/c a v1.1 debugger needs to be able
-// to look at a v2.0 app enough to recognize the version mismatch.
-// This check is only necessary for platforms that ran on v1.1 (Win32-x86)
-
-// Just to catch any potential illegal change in the IPC block, we assert the offsets against the offsets from v1.1.
-// The constants here are pulled from v1.1.
-// The RS will look at these versioning fields, so they absolutely must line up.
-static_assert_no_msg(offsetof(DebuggerIPCControlBlock, m_leftSideProtocolCurrent) == 0x10);
-static_assert_no_msg(offsetof(DebuggerIPCControlBlock, m_leftSideProtocolMinSupported) == 0x14);
-static_assert_no_msg(offsetof(DebuggerIPCControlBlock, m_rightSideProtocolCurrent) == 0x18);
-static_assert_no_msg(offsetof(DebuggerIPCControlBlock, m_rightSideProtocolMinSupported) == 0x1c);
-
-// Unfortunately, on detecting such failure, v1.1 will also null out LSEA, LSER and RSPH.
-// If these get adjusted, a version-mismatch attach  will effectively null out random fields.
-static_assert_no_msg(offsetof(DebuggerIPCControlBlock, m_paddingObsoleteLSEA) == 0x30);
-static_assert_no_msg(offsetof(DebuggerIPCControlBlock, m_paddingObsoleteLSER) == 0x34);
-static_assert_no_msg(offsetof(DebuggerIPCControlBlock, m_rightSideProcessHandle) == 0x38);
-
-
-
-#endif
-
 #define INITIAL_APP_DOMAIN_INFO_LIST_SIZE   16
 
 
@@ -847,18 +823,16 @@ typedef VMPTR_Base<DT_CONTEXT, PTR_CONTEXT> VMPTR_CONTEXT;
 typedef VMPTR_Base<DT_CONTEXT, void > VMPTR_CONTEXT;
 #endif
 
-// DomainFile is a base-class for a CLR module, with app-domain affinity.
-// For domain-neutral modules (like CoreLib), there is a DomainFile instance
+// DomainAssembly is a base-class for a CLR module, with app-domain affinity.
+// For domain-neutral modules (like CoreLib), there is a DomainAssembly instance
 // for each appdomain the module lives in.
 // This is the canonical handle ICorDebug uses to a CLR module.
-DEFINE_VMPTR(class DomainFile,      PTR_DomainFile,     VMPTR_DomainFile);
+DEFINE_VMPTR(class DomainAssembly,  PTR_DomainAssembly, VMPTR_DomainAssembly);
 DEFINE_VMPTR(class Module,          PTR_Module,         VMPTR_Module);
 
-// DomainAssembly derives from DomainFile and represents a manifest module.
-DEFINE_VMPTR(class DomainAssembly,  PTR_DomainAssembly, VMPTR_DomainAssembly);
 DEFINE_VMPTR(class Assembly,        PTR_Assembly,       VMPTR_Assembly);
 
-DEFINE_VMPTR(class PEFile,          PTR_PEFile,         VMPTR_PEFile);
+DEFINE_VMPTR(class PEAssembly,      PTR_PEAssembly,     VMPTR_PEAssembly);
 DEFINE_VMPTR(class MethodDesc,      PTR_MethodDesc,     VMPTR_MethodDesc);
 DEFINE_VMPTR(class FieldDesc,       PTR_FieldDesc,      VMPTR_FieldDesc);
 
@@ -990,7 +964,7 @@ extern const size_t nameCount;
 
 struct MSLAYOUT IPCENames // We use a class/struct so that the function can remain in a shared header file
 {
-    static DebuggerIPCEventType GetEventType(__in_z char * strEventType)
+    static DebuggerIPCEventType GetEventType(_In_z_ char * strEventType)
     {
         // pass in the string of event name and find the matching enum value
         // This is a linear search which is pretty slow. However, this is only used
@@ -1163,6 +1137,40 @@ struct MSLAYOUT DebuggerREGDISPLAY
     SIZE_T  LR;
     SIZE_T  SP;
     SIZE_T  PC;
+#elif defined(TARGET_LOONGARCH64)
+    #define DebuggerIPCE_FloatCount 32
+    SIZE_T  RA;
+    SIZE_T  TP;
+    SIZE_T  SP;
+    SIZE_T  A0;
+    SIZE_T  A1;
+    SIZE_T  A2;
+    SIZE_T  A3;
+    SIZE_T  A4;
+    SIZE_T  A5;
+    SIZE_T  A6;
+    SIZE_T  A7;
+    SIZE_T  T0;
+    SIZE_T  T1;
+    SIZE_T  T2;
+    SIZE_T  T3;
+    SIZE_T  T4;
+    SIZE_T  T5;
+    SIZE_T  T6;
+    SIZE_T  T7;
+    SIZE_T  T8;
+    SIZE_T  X0;
+    SIZE_T  FP;
+    SIZE_T  S0;
+    SIZE_T  S1;
+    SIZE_T  S2;
+    SIZE_T  S3;
+    SIZE_T  S4;
+    SIZE_T  S5;
+    SIZE_T  S6;
+    SIZE_T  S7;
+    SIZE_T  S8;
+    SIZE_T  PC;
 #else
     #define DebuggerIPCE_FloatCount 1
 
@@ -1276,7 +1284,7 @@ inline bool IsEqualOrCloserToRoot(FramePointer fp1, FramePointer fp2)
 //          the address of the real start address of the native code.
 //          This field will be NULL only if the method hasn't been JITted
 //          yet (and thus no code is available).  Otherwise, it will be
-//          the adress of a CORDB_ADDRESS in the remote memory.  This
+//          the address of a CORDB_ADDRESS in the remote memory.  This
 //          CORDB_ADDRESS may be NULL, in which case the code is unavailable
 //          has been pitched (return CORDBG_E_CODE_NOT_AVAILABLE)
 //
@@ -1285,7 +1293,7 @@ inline bool IsEqualOrCloserToRoot(FramePointer fp1, FramePointer fp2)
 struct MSLAYOUT DebuggerIPCE_FuncData
 {
     mdMethodDef funcMetadataToken;
-    VMPTR_DomainFile vmDomainFile;
+    VMPTR_DomainAssembly vmDomainAssembly;
 
     mdTypeDef   classMetadataToken;
 
@@ -1325,7 +1333,7 @@ struct MSLAYOUT DebuggerIPCE_FuncData
 //          generic code of some kind)
 // BOOL isInstantiatedGeneric: Indicates if the method is
 //          generic code of some kind.
-// BOOL jsutAfterILThrow: indicates that code just threw a software exception and
+// BOOL justAfterILThrow: indicates that code just threw a software exception and
 //          nativeOffset points to an instruction just after [call IL_Throw].
 //          This is being used to figure out a real offset of the exception origin.
 //          By subtracting STACKWALK_CONTROLPC_ADJUST_OFFSET from nativeOffset you can get
@@ -1359,7 +1367,7 @@ struct MSLAYOUT DebuggerIPCE_JITFuncData
     // this is the version of the jitted code
     SIZE_T       enCVersion;
 
-    BOOL         jsutAfterILThrow;
+    BOOL         justAfterILThrow;
 };
 
 //
@@ -1425,7 +1433,7 @@ struct MSLAYOUT DebuggerIPCE_STRData
         struct MSLAYOUT
         {
             mdMethodDef funcMetadataToken;
-            VMPTR_DomainFile vmDomainFile;
+            VMPTR_DomainAssembly vmDomainAssembly;
             VMPTR_MethodDesc vmMethodDesc;
             CorDebugInternalFrameType frameType;
         } stubFrame;
@@ -1469,7 +1477,7 @@ struct MSLAYOUT DebuggerIPCE_BasicTypeData
     CorElementType  elementType;
     mdTypeDef       metadataToken;
     VMPTR_Module     vmModule;
-    VMPTR_DomainFile vmDomainFile;
+    VMPTR_DomainAssembly vmDomainAssembly;
     VMPTR_TypeHandle vmTypeHandle;
 };
 
@@ -1505,7 +1513,7 @@ struct MSLAYOUT DebuggerIPCE_ExpandedTypeData
          {
             mdTypeDef       metadataToken;
             VMPTR_Module vmModule;
-            VMPTR_DomainFile vmDomainFile;
+            VMPTR_DomainAssembly vmDomainAssembly;
             VMPTR_TypeHandle typeHandle; // if non-null then further fetches will be needed to get type arguments
         } ClassTypeData;
 
@@ -1679,7 +1687,7 @@ struct MSLAYOUT DebuggerIPCE_FuncEvalInfo
     DebuggerIPCE_FuncEvalType  funcEvalType;
     mdMethodDef                funcMetadataToken;
     mdTypeDef                  funcClassMetadataToken;
-    VMPTR_DomainFile           vmDomainFile;
+    VMPTR_DomainAssembly       vmDomainAssembly;
     RSPTR_CORDBEVAL            funcEvalKey;
     bool                       evalDuringException;
 
@@ -1880,6 +1888,13 @@ C_ASSERT(DBG_TARGET_REGNUM_AMBIENT_SP == ICorDebugInfo::REGNUM_AMBIENT_SP);
 C_ASSERT(DBG_TARGET_REGNUM_SP == ICorDebugInfo::REGNUM_SP);
 C_ASSERT(DBG_TARGET_REGNUM_AMBIENT_SP == ICorDebugInfo::REGNUM_AMBIENT_SP);
 #endif // TARGET_ARM64
+#elif defined(TARGET_LOONGARCH64)
+#define DBG_TARGET_REGNUM_SP 3
+#define DBG_TARGET_REGNUM_AMBIENT_SP 34
+#ifdef TARGET_LOONGARCH64
+C_ASSERT(DBG_TARGET_REGNUM_SP == ICorDebugInfo::REGNUM_SP);
+C_ASSERT(DBG_TARGET_REGNUM_AMBIENT_SP == ICorDebugInfo::REGNUM_AMBIENT_SP);
+#endif
 #else
 #error Target registers are not defined for this platform
 #endif
@@ -1915,9 +1930,9 @@ struct MSLAYOUT DebuggerIPCEvent
 
         struct MSLAYOUT
         {
-            // Module whos metadata is being updated
+            // Module whose metadata is being updated
             // This tells the RS that the metadata for that module has become invalid.
-            VMPTR_DomainFile vmDomainFile;
+            VMPTR_DomainAssembly vmDomainAssembly;
 
         } MetadataUpdateData;
 
@@ -1960,13 +1975,13 @@ struct MSLAYOUT DebuggerIPCEvent
         struct MSLAYOUT
         {
             // Module that was just loaded.
-            VMPTR_DomainFile vmDomainFile;
+            VMPTR_DomainAssembly vmDomainAssembly;
         }LoadModuleData;
 
 
         struct MSLAYOUT
         {
-            VMPTR_DomainFile vmDomainFile;
+            VMPTR_DomainAssembly vmDomainAssembly;
             LSPTR_ASSEMBLY debuggerAssemblyToken;
         } UnloadModuleData;
 
@@ -1975,7 +1990,7 @@ struct MSLAYOUT DebuggerIPCEvent
         // Queury PDB from OOP
         struct MSLAYOUT
         {
-            VMPTR_DomainFile vmDomainFile;
+            VMPTR_DomainAssembly vmDomainAssembly;
         } UpdateModuleSymsData;
 
         DebuggerMDANotification MDANotification;
@@ -1984,7 +1999,7 @@ struct MSLAYOUT DebuggerIPCEvent
         {
             LSPTR_BREAKPOINT breakpointToken;
             mdMethodDef  funcMetadataToken;
-            VMPTR_DomainFile vmDomainFile;
+            VMPTR_DomainAssembly vmDomainAssembly;
             bool         isIL;
             SIZE_T       offset;
             SIZE_T       encVersion;
@@ -2064,7 +2079,7 @@ struct MSLAYOUT DebuggerIPCEvent
         // Apply an EnC edit
         struct MSLAYOUT
         {
-            VMPTR_DomainFile vmDomainFile;      // Module to edit
+            VMPTR_DomainAssembly vmDomainAssembly;      // Module to edit
             DWORD cbDeltaMetadata;              // size of blob pointed to by pDeltaMetadata
             CORDB_ADDRESS pDeltaMetadata;       // pointer to delta metadata in debuggee
                                                 // it's the RS's responsibility to allocate and free
@@ -2081,20 +2096,20 @@ struct MSLAYOUT DebuggerIPCEvent
         struct MSLAYOUT
         {
             mdTypeDef   classMetadataToken;
-            VMPTR_DomainFile vmDomainFile;
+            VMPTR_DomainAssembly vmDomainAssembly;
             LSPTR_ASSEMBLY classDebuggerAssemblyToken;
         } LoadClass;
 
         struct MSLAYOUT
         {
             mdTypeDef   classMetadataToken;
-            VMPTR_DomainFile vmDomainFile;
+            VMPTR_DomainAssembly vmDomainAssembly;
             LSPTR_ASSEMBLY classDebuggerAssemblyToken;
         } UnloadClass;
 
         struct MSLAYOUT
         {
-            VMPTR_DomainFile vmDomainFile;
+            VMPTR_DomainAssembly vmDomainAssembly;
             bool  flag;
         } SetClassLoad;
 
@@ -2125,7 +2140,7 @@ struct MSLAYOUT DebuggerIPCEvent
             CORDB_ADDRESS    startAddress;
             bool             fCanSetIPOnly;
             VMPTR_Thread     vmThreadToken;
-            VMPTR_DomainFile vmDomainFile;
+            VMPTR_DomainAssembly vmDomainAssembly;
             mdMethodDef      mdMethod;
             VMPTR_MethodDesc vmMethodDesc;
             SIZE_T           offset;
@@ -2154,7 +2169,7 @@ struct MSLAYOUT DebuggerIPCEvent
         struct MSLAYOUT
         {
             // Domain file for the domain in which the notification occurred
-            VMPTR_DomainFile vmDomainFile;
+            VMPTR_DomainAssembly vmDomainAssembly;
 
             // metadata token for the type of the CustomNotification object's type
             mdTypeDef    classToken;
@@ -2219,7 +2234,7 @@ struct MSLAYOUT DebuggerIPCEvent
 
         struct MSLAYOUT
         {
-            VMPTR_DomainFile vmDomainFile;
+            VMPTR_DomainAssembly vmDomainAssembly;
             BOOL             fAllowJitOpts;
             BOOL             fEnableEnC;
         } JitDebugInfo;
@@ -2227,7 +2242,7 @@ struct MSLAYOUT DebuggerIPCEvent
         // EnC Remap opportunity
         struct MSLAYOUT
         {
-            VMPTR_DomainFile vmDomainFile;
+            VMPTR_DomainAssembly vmDomainAssembly;
             mdMethodDef funcMetadataToken ;        // methodDef of function with remap opportunity
             SIZE_T          currentVersionNumber;  // version currently executing
             SIZE_T          resumeVersionNumber;   // latest version
@@ -2239,7 +2254,7 @@ struct MSLAYOUT DebuggerIPCEvent
         // EnC Remap has taken place
         struct MSLAYOUT
         {
-            VMPTR_DomainFile vmDomainFile;
+            VMPTR_DomainAssembly vmDomainAssembly;
             mdMethodDef funcMetadataToken;         // methodDef of function that was remapped
         } EnCRemapComplete;
 
@@ -2247,7 +2262,7 @@ struct MSLAYOUT DebuggerIPCEvent
         // specific edit made by EnC (function add/update or field add).
         struct MSLAYOUT
         {
-            VMPTR_DomainFile vmDomainFile;
+            VMPTR_DomainAssembly vmDomainAssembly;
             mdToken         memberMetadataToken;   // Either a methodDef token indicating the function that
                                                    // was updated/added, or a fieldDef token indicating the
                                                    // field which was added.
@@ -2268,7 +2283,7 @@ struct MSLAYOUT DebuggerIPCEvent
         // @todo - Perhaps we can bundle these up so we can set multiple funcs w/ 1 event?
         struct MSLAYOUT
         {
-            VMPTR_DomainFile vmDomainFile;
+            VMPTR_DomainAssembly vmDomainAssembly;
             mdMethodDef     funcMetadataToken;
             DWORD           dwStatus;
         } SetJMCFunctionStatus;
