@@ -27,7 +27,7 @@
 #include "ex.h"
 #include "sarray.h"
 #include "memoryrange.h"
-#include "../md/hotdata/hotheap.h"
+#include "../md/datablob.h"
 
 //*****************************************************************************
 // NOTE:
@@ -49,7 +49,6 @@ const int DFT_CODE_HEAP_SIZE = 8192;
 class StgStringPool;
 class StgBlobPool;
 class StgCodePool;
-class CorProfileData;
 
 //  Perform binary search on index table.
 //
@@ -97,7 +96,7 @@ protected:
     BYTE       *m_pSegData;     // Pointer to the data.
     StgPoolSeg *m_pNextSeg;     // Pointer to next segment, or NULL.
     // Size of the segment buffer. If this is last segment (code:m_pNextSeg is NULL), then it's the
-    // allocation size. If this is not the last segment, then this is shrinked to segment data size
+    // allocation size. If this is not the last segment, then this is shrunk to segment data size
     // (code:m_cbSegNext).
     ULONG       m_cbSegSize;
     ULONG       m_cbSegNext;    // Offset of next available byte in segment.
@@ -203,7 +202,7 @@ public:
     __checkReturn
     inline HRESULT GetString(
                     UINT32  nIndex,
-        __deref_out LPCSTR *pszString)
+        _Outptr_ LPCSTR *pszString)
     {
         HRESULT hr;
 
@@ -234,12 +233,12 @@ public:
 
 //*****************************************************************************
 // Return a pointer to a null terminated string given an offset previously
-// handed out by AddString or FindString. Only valid for use if the Storage pool is actuall ReadOnly, and not derived
+// handed out by AddString or FindString. Only valid for use if the Storage pool is actually ReadOnly, and not derived
 //*****************************************************************************
     __checkReturn
     inline HRESULT GetStringReadOnly(
                     UINT32  nIndex,
-        __deref_out LPCSTR *pszString)
+        _Outptr_ LPCSTR *pszString)
     {
         HRESULT hr;
 
@@ -277,7 +276,7 @@ public:
     __checkReturn
     virtual HRESULT GetStringW(                         // Return code.
         ULONG                          iOffset,         // Offset of string in pool.
-        __out_ecount(cchBuffer) LPWSTR szOut,           // Output buffer for string.
+        _Out_writes_(cchBuffer) LPWSTR szOut,           // Output buffer for string.
         int                            cchBuffer);      // Size of output buffer.
 
 //*****************************************************************************
@@ -335,22 +334,6 @@ public:
         UINT32              nOffset,    // Offset of blob in pool.
         MetaData::DataBlob *pData);
 
-#ifdef FEATURE_PREJIT
-    // Initialize hot data structures.
-    // Method can be called multiple time, e.g. to disable usage of hot data structures in certain scenarios
-    // (see code:CMiniMd::DisableHotDataUsage).
-    void InitHotData(MetaData::HotHeap hotHeap)
-    {
-        LIMITED_METHOD_CONTRACT;
-
-#if !defined(FEATURE_UTILCODE_NO_DEPENDENCIES)
-        m_HotHeap = hotHeap;
-#else
-        _ASSERTE(!"InitHotData(): Not supposed to exist in RoMetaData.dll");
-#endif //!(defined(FEATURE_UTILCODE_NO_DEPENDENCIES))
-    }
-#endif //FEATURE_PREJIT
-
 protected:
 
 //*****************************************************************************
@@ -377,21 +360,6 @@ protected:
             return CLDB_E_INDEX_NOTFOUND;
         }
 
-#if !defined(FEATURE_UTILCODE_NO_DEPENDENCIES)
-#ifdef FEATURE_PREJIT
-        // try hot data first
-        if (!m_HotHeap.IsEmpty())
-        {
-            HRESULT hr = m_HotHeap.GetData(nOffset, pData);
-            if ((hr == S_OK) || FAILED(hr))
-            {
-                return hr;
-            }
-            _ASSERTE(hr == S_FALSE);
-        }
-#endif //FEATURE_PREJIT
-#endif //!(defined(FEATURE_UTILCODE_NO_DEPENDENCIES))
-
 
         pData->Init(m_pSegData + nOffset, m_cbSegSize - nOffset);
 
@@ -410,13 +378,6 @@ protected:
         WRAPPER_NO_CONTRACT;
         return GetDataReadOnly(nOffset, pData);
     } // StgPoolReadOnly::GetData
-
-private:
-#if !defined(FEATURE_UTILCODE_NO_DEPENDENCIES)
-    // hot pool data
-    MetaData::HotHeap m_HotHeap;
-#endif //!(defined(FEATURE_UTILCODE_NO_DEPENDENCIES))
-
 };  // class StgPoolReadOnly
 
 //
@@ -833,7 +794,7 @@ public:
 //*****************************************************************************
 // Load a string heap from persisted memory.  If a copy of the data is made
 // (so that it may be updated), then a new hash table is generated which can
-// be used to elminate duplicates with new strings.
+// be used to eliminate duplicates with new strings.
 //*****************************************************************************
     __checkReturn
     HRESULT InitOnMem(                        // Return code.
@@ -990,7 +951,7 @@ public:
 //*****************************************************************************
 // Load a Guid heap from persisted memory.  If a copy of the data is made
 // (so that it may be updated), then a new hash table is generated which can
-// be used to elminate duplicates with new Guids.
+// be used to eliminate duplicates with new Guids.
 //*****************************************************************************
     __checkReturn
     HRESULT InitOnMem(                      // Return code.
@@ -1420,7 +1381,7 @@ private:
 
 //*****************************************************************************
 // CGrowableStream is a simple IStream implementation that grows as
-// its written to. All the memory is contigious, so read access is
+// its written to. All the memory is contiguous, so read access is
 // fast. A grow does a realloc, so be aware of that if you're going to
 // use this.
 //*****************************************************************************
