@@ -369,7 +369,7 @@ ErrExit:
 //*****************************************************************************
 HRESULT
 Disp::GetCORSystemDirectory(
-    __out_ecount (cchBuffer) LPWSTR szBuffer,      // [out] Buffer for the directory name
+    _Out_writes_ (cchBuffer) LPWSTR szBuffer,      // [out] Buffer for the directory name
     DWORD                           cchBuffer,     // [in] Size of the buffer
     DWORD                          *pcchBuffer)    // [out] Number of characters returned
 {
@@ -387,7 +387,7 @@ HRESULT Disp::FindAssembly(             // S_OK or error
     LPCWSTR     szAssemblyName,         // [IN] required - this is the assembly you are requesting
     LPCWSTR     szName,                 // [OUT] buffer - to hold name
     ULONG       cchName,                // [IN] the name buffer's size
-    ULONG       *pcName)                // [OUT] the number of characters returend in the buffer
+    ULONG       *pcName)                // [OUT] the number of characters returned in the buffer
 {
     BEGIN_ENTRYPOINT_NOTHROW;
     END_ENTRYPOINT_NOTHROW;
@@ -401,9 +401,9 @@ HRESULT Disp::FindAssemblyModule(           // S_OK or error
     LPCWSTR     szGlobalBin,                // [IN] optional - can be NULL
     LPCWSTR     szAssemblyName,             // [IN] The assembly name or code base of the assembly
     LPCWSTR     szModuleName,               // [IN] required - the name of the module
-    __out_ecount (cchName) LPWSTR  szName,  // [OUT] buffer - to hold name
+    _Out_writes_ (cchName) LPWSTR  szName,  // [OUT] buffer - to hold name
     ULONG       cchName,                    // [IN]  the name buffer's size
-    ULONG       *pcName)                    // [OUT] the number of characters returend in the buffer
+    ULONG       *pcName)                    // [OUT] the number of characters returned in the buffer
 {
     BEGIN_ENTRYPOINT_NOTHROW;
     END_ENTRYPOINT_NOTHROW;
@@ -430,7 +430,7 @@ HRESULT Disp::OpenScopeOnITypeInfo(     // Return code.
 //*****************************************************************************
 // Create a brand new scope which will be used for portable PDB metadata.
 // This is based on the CLSID that was used to get the dispenser.
-// 
+//
 // The existing DefineScope method cannot be used for the purpose of PDB
 // metadata generation, since it internally creates module and type def table
 // entries.
@@ -445,7 +445,7 @@ Disp::DefinePortablePdbScope(
 {
 #ifdef FEATURE_METADATA_EMIT
     HRESULT     hr = S_OK;
-    
+
     BEGIN_ENTRYPOINT_NOTHROW;
 
     RegMeta* pMeta = 0;
@@ -457,7 +457,7 @@ Disp::DefinePortablePdbScope(
         IfFailGo(E_INVALIDARG);
 
     // Currently the portable PDB tables are treated as an extension to the MDVersion2
-    // TODO: this extension might deserve its own version number e.g. 'MDVersion3'  
+    // TODO: this extension might deserve its own version number e.g. 'MDVersion3'
     if (rclsid == CLSID_CLR_v2_MetaData)
     {
         optionForNewScope.m_MetadataVersion = MDVersion2;
@@ -876,7 +876,7 @@ HRESULT Disp::GetOption(                // Return code.
     {   // Note: This is not used in CLR sources anymore, but we store the value and return it here,
         // so we keep it for backward-compat.
         V_VT(pvalue) = VT_BOOL;
-        V_BOOL(pvalue) = m_OptionValue.m_GenerateTCEAdapters;
+        V_BOOL(pvalue) = !!m_OptionValue.m_GenerateTCEAdapters ? VARIANT_TRUE : VARIANT_FALSE;
     }
 #endif //FEATURE_METADATA_EMIT_ALL || FEATURE_METADATA_EMIT_IN_DEBUGGER
     else
