@@ -67,7 +67,6 @@ RegMeta::RegMeta() :
     m_SetAPICaller(EXTERNAL_CALLER),
     m_ModuleType(ValidatorModuleTypeInvalid),
     m_bKeepKnownCa(false),
-    m_pCorProfileData(NULL),
     m_ReorderingOptions(NoReordering)
 #ifdef FEATURE_METADATA_RELEASE_MEMORY_ON_REOPEN
     , m_safeToDeleteStgdb(true)
@@ -668,16 +667,6 @@ RegMeta::QueryInterface(
         }
     }
 #endif // FEATURE_COMINTEROP
-#ifdef FEATURE_PREJIT
-    else if (riid == IID_IMetaDataCorProfileData)
-    {
-        *ppUnk = (IMetaDataCorProfileData *)this;
-    }
-    else if (riid == IID_IMDInternalMetadataReorderingOptions)
-    {
-        *ppUnk = (IMDInternalMetadataReorderingOptions *)this;
-    }
-#endif //FEATURE_PREJIT
 #endif //FEATURE_METADATA_IN_VM
     else
     {
@@ -821,20 +810,20 @@ ErrExit:
 #define STRING_BUFFER_LEN 1024
 #define ENUM_BUFFER_SIZE 10
 
-int DumpMD_Write(__in __in_z const char *str)
+int DumpMD_Write(_In_ _In_z_ const char *str)
 {
     OutputDebugStringA(str);
     return 0; // strlen(str);
 } // int DumpMD_Write()
 
-int DumpMD_WriteLine(__in __in_z const char *str)
+int DumpMD_WriteLine(_In_ _In_z_ const char *str)
 {
     OutputDebugStringA(str);
     OutputDebugStringA("\n");
     return 0; // strlen(str);
 } // int DumpMD_Write()
 
-int DumpMD_VWriteMarker(__in __in_z const char *str, va_list marker)
+int DumpMD_VWriteMarker(_In_ _In_z_ const char *str, va_list marker)
 {
     CQuickBytes m_output;
 
@@ -856,7 +845,7 @@ int DumpMD_VWriteMarker(__in __in_z const char *str, va_list marker)
     return count;
 } // int DumpMD_VWriteMarker()
 
-int DumpMD_VWrite(__in __in_z const char *str, ...)
+int DumpMD_VWrite(_In_ _In_z_ const char *str, ...)
 {
     va_list marker;
     int     count;
@@ -867,7 +856,7 @@ int DumpMD_VWrite(__in __in_z const char *str, ...)
     return count;
 } // int DumpMD_VWrite()
 
-int DumpMD_VWriteLine(__in __in_z const char *str, ...)
+int DumpMD_VWriteLine(_In_ _In_z_ const char *str, ...)
 {
     va_list marker;
     int     count;
@@ -920,7 +909,7 @@ const char *DumpMD_DumpRawNameOfType(RegMeta *pMD, ULONG iType)
     }
     // default:
     static char buf[30];
-    sprintf_s(buf, NumItems(buf), "unknown type 0x%02x", iType);
+    sprintf_s(buf, ARRAY_SIZE(buf), "unknown type 0x%02x", iType);
     return buf;
 } // const char *DumpMD_DumpRawNameOfType()
 
@@ -1137,7 +1126,7 @@ void DumpMD_DisplayUserStrings(
     bool        bUnprint = false;       // Is an unprintable character found?
     HRESULT     hr;                     // A result.
     while (SUCCEEDED(hr = pMD->EnumUserStrings( &stringEnum,
-                             Strings, NumItems(Strings), &count)) &&
+                             Strings, ARRAY_SIZE(Strings), &count)) &&
             count > 0)
     {
         if (totalCount == 1)
@@ -1235,7 +1224,7 @@ void DumpMD_DumpRawHeaps(
     do
     {
         pMD->GetBlob(oData, &cbData, (const void**)&pData);
-        sprintf_s(rcPrefix, NumItems(rcPrefix), "%5x,%-2x", oData, cbData);
+        sprintf_s(rcPrefix, ARRAY_SIZE(rcPrefix), "%5x,%-2x", oData, cbData);
         DumpMD_DumpHex(rcPrefix, pData, cbData);
         hr = pMD->GetNextBlob(oData, &oData);
     }
@@ -1249,7 +1238,7 @@ void DumpMD_DumpRawHeaps(
     do
     {
         pMD->GetString(oData, &pString);
-        sprintf_s(rcPrefix, NumItems(rcPrefix), "%08x", oData);
+        sprintf_s(rcPrefix, ARRAY_SIZE(rcPrefix), "%08x", oData);
         DumpMD_DumpHex(rcPrefix, pString, (ULONG)strlen(pString)+1);
         if (*pString != 0)
             DumpMD_VWrite("%08x: %s\n", oData, pString);
