@@ -61,7 +61,7 @@ LIST_ENTRY MappedViewList;
 #ifndef CORECLR
 static PAL_ERROR MAPCreateTempFile(CPalThread *, PINT, PSZ);
 #endif // !CORECLR
-static PAL_ERROR MAPGrowLocalFile(INT, UINT);
+static PAL_ERROR MAPGrowLocalFile(INT, off_t);
 static PMAPPED_VIEW_LIST MAPGetViewForAddress( LPCVOID );
 static PAL_ERROR MAPDesiredAccessAllowed( DWORD, DWORD, DWORD );
 
@@ -389,6 +389,7 @@ CorUnix::InternalCreateFileMapping(
     INT UnixFd = -1;
     BOOL bPALCreatedTempFile = FALSE;
     UINT nFileSize = 0;
+    off_t maximumSize;
 
     //
     // Validate parameters
@@ -401,13 +402,6 @@ CorUnix::InternalCreateFileMapping(
         goto ExitInternalCreateFileMapping;
     }
 
-    if (0 != dwMaximumSizeHigh)
-    {
-        ASSERT("dwMaximumSizeHigh is always 0.\n");
-        palError = ERROR_INVALID_PARAMETER;
-        goto ExitInternalCreateFileMapping;
-    }
-
     if (PAGE_READWRITE != flProtect
         && PAGE_READONLY != flProtect
         && PAGE_WRITECOPY != flProtect)
@@ -419,7 +413,8 @@ CorUnix::InternalCreateFileMapping(
         goto ExitInternalCreateFileMapping;
     }
 
-    if (hFile == INVALID_HANDLE_VALUE && 0 == dwMaximumSizeLow)
+    if (hFile == INVALID_HANDLE_VALUE &&
+        0 == dwMaximumSizeLow && 0 == dwMaximumSizeHigh)
     {
         ERROR( "If hFile is INVALID_HANDLE_VALUE, then you must specify a size.\n" );
         palError = ERROR_INVALID_PARAMETER;
@@ -433,6 +428,8 @@ CorUnix::InternalCreateFileMapping(
         goto ExitInternalCreateFileMapping;
     }
 
+    maximumSize = ((off_t)dwMaximumSizeHigh << 32) | (off_t)dwMaximumSizeLow;
+
     palError = g_pObjectManager->AllocateObject(
         pThread,
         &otFileMapping,
@@ -597,8 +594,7 @@ CorUnix::InternalCreateFileMapping(
             goto ExitInternalCreateFileMapping;
         }
 
-        if ( 0 == UnixFileInformation.st_size &&
-             0 == dwMaximumSizeHigh && 0 == dwMaximumSizeLow )
+        if ( 0 == UnixFileInformation.st_size && 0 == maximumSize )
         {
             ERROR( "The file cannot be a zero length file.\n" );
             palError = ERROR_FILE_INVALID;
@@ -606,7 +602,7 @@ CorUnix::InternalCreateFileMapping(
         }
 
         if ( INVALID_HANDLE_VALUE != hFile &&
-             dwMaximumSizeLow > (DWORD) UnixFileInformation.st_size &&
+             maximumSize > UnixFileInformation.st_size &&
              ( PAGE_READONLY == flProtect || PAGE_WRITECOPY == flProtect ) )
         {
             /* In this situation, Windows returns an error, because the
@@ -616,12 +612,12 @@ CorUnix::InternalCreateFileMapping(
             goto ExitInternalCreateFileMapping;
         }
 
-        if ( (DWORD) UnixFileInformation.st_size < dwMaximumSizeLow )
+        if ( UnixFileInformation.st_size < maximumSize )
         {
             TRACE( "Growing the size of file on disk to match requested size.\n" );
 
             /* Need to grow the file on disk to match size. */
-            palError = MAPGrowLocalFile(UnixFd, dwMaximumSizeLow);
+            palError = MAPGrowLocalFile(UnixFd, maximumSize);
             if (NO_ERROR != palError)
             {
                 ERROR( "Unable to grow the file on disk.\n" );
@@ -630,8 +626,8 @@ CorUnix::InternalCreateFileMapping(
         }
     }
 
-    nFileSize = ( 0 == dwMaximumSizeLow && 0 == dwMaximumSizeHigh ) ?
-        UnixFileInformation.st_size : dwMaximumSizeLow;
+    nFileSize = ( 0 == maximumSize ) ?
+        UnixFileInformation.st_size : maximumSize;
 
     pImmutableData->MaxSize = nFileSize;
     pImmutableData->flProtect = flProtect;
@@ -1589,7 +1585,7 @@ Function :
     Grows the file on disk to match the specified size.
 
 --*/
-static PAL_ERROR MAPGrowLocalFile( INT UnixFD, UINT NewSize )
+static PAL_ERROR MAPGrowLocalFile( INT UnixFD, off_t NewSize )
 {
     PAL_ERROR palError = NO_ERROR;
     INT  TruncateRetVal = -1;
@@ -1604,11 +1600,11 @@ static PAL_ERROR MAPGrowLocalFile( INT UnixFD, UINT NewSize )
 
     /* ftruncate is a standard function, but the behavior of enlarging files is
     non-standard.  So I will try to enlarge a file, and if that fails try the
-    less efficent way.*/
+    less efficient way.*/
     TruncateRetVal = ftruncate( UnixFD, NewSize );
     fstat( UnixFD, &FileInfo );
 
-    if ( TruncateRetVal != 0 || FileInfo.st_size != (int) NewSize )
+    if ( TruncateRetVal != 0 || FileInfo.st_size != NewSize )
     {
         INT OrigSize;
         CONST UINT  BUFFER_SIZE = 128;
@@ -1616,7 +1612,7 @@ static PAL_ERROR MAPGrowLocalFile( INT UnixFD, UINT NewSize )
         UINT x = 0;
         UINT CurrentPosition = 0;
 
-        TRACE( "Trying the less efficent way.\n" );
+        TRACE( "Trying the less efficient way.\n" );
 
         CurrentPosition = lseek( UnixFD, 0, SEEK_CUR );
         OrigSize = lseek( UnixFD, 0, SEEK_END );
@@ -2079,9 +2075,9 @@ MAPmmapAndRecord(
         // Set the requested mapping with forced PROT_WRITE to ensure data from the file can be read there,
         // read the data in and finally remove the forced PROT_WRITE. On Intel we can still switch the
         // protection later with mprotect.
-        if ((mprotect(pvBaseAddress, len + adjust, prot | PROT_WRITE) == -1) ||
+        if ((mprotect(pvBaseAddress, len + adjust, PROT_WRITE) == -1) ||
             (pread(fd, pvBaseAddress, len + adjust, offset - adjust) == -1) ||
-            (((prot & PROT_WRITE) == 0) && mprotect(pvBaseAddress, len + adjust, prot) == -1))
+            (mprotect(pvBaseAddress, len + adjust, prot) == -1))
         {
             palError = FILEGetLastErrorFromErrno();
         }
@@ -2216,7 +2212,7 @@ void * MAPMapPEFile(HANDLE hFile, off_t offset)
     //don't need more directories.
 
     //I now know how big the file is.  Reserve enough address space for the whole thing.  Try to get the
-    //preferred base.  Create the intial mapping as "no access".  We'll use that for the guard pages in the
+    //preferred base.  Create the initial mapping as "no access".  We'll use that for the guard pages in the
     //"holes" between sections.
     SIZE_T preferredBase, virtualSize;
     preferredBase = ntHeader.OptionalHeader.ImageBase;
@@ -2288,7 +2284,13 @@ void * MAPMapPEFile(HANDLE hFile, off_t offset)
     // more efficient code (by avoiding usage of jump stubs). Alignment to a 64 KB granularity should
     // not be necessary (alignment to page size should be sufficient), but see
     // ExecutableMemoryAllocator::AllocateMemory() for the reason why it is done.
-    loadedBase = ReserveMemoryFromExecutableAllocator(pThread, ALIGN_UP(reserveSize, VIRTUAL_64KB));
+
+#ifdef FEATURE_ENABLE_NO_ADDRESS_SPACE_RANDOMIZATION
+    if (!g_useDefaultBaseAddr)
+#endif // FEATURE_ENABLE_NO_ADDRESS_SPACE_RANDOMIZATION
+    {
+        loadedBase = ReserveMemoryFromExecutableAllocator(pThread, ALIGN_UP(reserveSize, VIRTUAL_64KB));
+    }
 #endif // HOST_64BIT
 
     if (loadedBase == NULL)
@@ -2398,8 +2400,9 @@ void * MAPMapPEFile(HANDLE hFile, off_t offset)
         goto doneReleaseMappingCriticalSection;
     }
 
-    void* prevSectionEnd;
-    prevSectionEnd = (char*)loadedHeader + headerSize; // the first "section" for our purposes is the header
+    void* prevSectionEndAligned;
+    // the first "section" for our purposes is the header
+    prevSectionEndAligned = ALIGN_UP((char*)loadedHeader + headerSize, GetVirtualPageSize());
 
     for (unsigned i = 0; i < numSections; ++i)
     {
@@ -2416,7 +2419,7 @@ void * MAPMapPEFile(HANDLE hFile, off_t offset)
         if (   (sectionBase < loadedHeader)                                                           // Did computing the section base overflow?
             || ((char*)sectionBase + currentHeader.SizeOfRawData < (char*)sectionBase)              // Does the section overflow?
             || ((char*)sectionBase + currentHeader.SizeOfRawData > (char*)loadedHeader + virtualSize) // Does the section extend past the end of the image as the header stated?
-            || (prevSectionEnd > sectionBase)                                                       // Does this section overlap the previous one?
+            || (prevSectionEndAligned > sectionBase)                                                       // Does this section overlap the previous one?
             )
         {
             ERROR_(LOADER)( "section %d is corrupt\n", i );
@@ -2439,12 +2442,12 @@ void * MAPMapPEFile(HANDLE hFile, off_t offset)
         }
 
         // Is there space between the previous section and this one? If so, add a PROT_NONE mapping to cover it.
-        if (prevSectionEnd < sectionBaseAligned)
+        if (prevSectionEndAligned < sectionBaseAligned)
         {
             palError = MAPRecordMapping(pFileObject,
                             loadedBase,
-                            prevSectionEnd,
-                            (char*)sectionBaseAligned - (char*)prevSectionEnd,
+                            prevSectionEndAligned,
+                            (char*)sectionBaseAligned - (char*)prevSectionEndAligned,
                             PROT_NONE);
             if (NO_ERROR != palError)
             {
@@ -2492,18 +2495,18 @@ void * MAPMapPEFile(HANDLE hFile, off_t offset)
         }
 #endif // _DEBUG
 
-        prevSectionEnd = ALIGN_UP((char*)sectionBase + currentHeader.SizeOfRawData, GetVirtualPageSize()); // round up to page boundary
+        prevSectionEndAligned = ALIGN_UP((char*)sectionBase + currentHeader.SizeOfRawData, GetVirtualPageSize()); // round up to page boundary
     }
 
     // Is there space after the last section and before the end of the mapped image? If so, add a PROT_NONE mapping to cover it.
     char* imageEnd;
     imageEnd = (char*)loadedBase + virtualSize; // actually, points just after the mapped end
-    if (prevSectionEnd < imageEnd)
+    if (prevSectionEndAligned < imageEnd)
     {
         palError = MAPRecordMapping(pFileObject,
                         loadedBase,
-                        prevSectionEnd,
-                        offset + (char*)imageEnd - (char*)prevSectionEnd,
+                        prevSectionEndAligned,
+                        offset + (char*)imageEnd - (char*)prevSectionEndAligned,
                         PROT_NONE);
         if (NO_ERROR != palError)
         {
