@@ -4,21 +4,103 @@
 #include "pedecoder.h"
 #include "executableallocator.h"
 
-#if USE_UPPER_ADDRESS
+#if USE_LAZY_PREFERRED_RANGE
 // Preferred region to allocate the code in.
-BYTE * ExecutableAllocator::g_codeMinAddr;
-BYTE * ExecutableAllocator::g_codeMaxAddr;
-BYTE * ExecutableAllocator::g_codeAllocStart;
+BYTE * ExecutableAllocator::g_lazyPreferredRangeStart;
 // Next address to try to allocate for code in the preferred region.
-BYTE * ExecutableAllocator::g_codeAllocHint;
-#endif // USE_UPPER_ADDRESS
+BYTE * ExecutableAllocator::g_lazyPreferredRangeHint;
+#endif // USE_LAZY_PREFERRED_RANGE
+
+BYTE * ExecutableAllocator::g_preferredRangeMin;
+BYTE * ExecutableAllocator::g_preferredRangeMax;
 
 bool ExecutableAllocator::g_isWXorXEnabled = false;
 
 ExecutableAllocator::FatalErrorHandler ExecutableAllocator::g_fatalErrorHandler = NULL;
-
 ExecutableAllocator* ExecutableAllocator::g_instance = NULL;
 
+#ifdef LOG_EXECUTABLE_ALLOCATOR_STATISTICS
+int64_t ExecutableAllocator::g_mapTimeSum = 0;
+int64_t ExecutableAllocator::g_mapTimeWithLockSum = 0;
+int64_t ExecutableAllocator::g_unmapTimeSum = 0;
+int64_t ExecutableAllocator::g_unmapTimeWithLockSum = 0;
+int64_t ExecutableAllocator::g_mapFindRXTimeSum = 0;
+int64_t ExecutableAllocator::g_mapCreateTimeSum = 0;
+int64_t ExecutableAllocator::g_releaseCount = 0;
+int64_t ExecutableAllocator::g_reserveCount = 0;
+
+ExecutableAllocator::LogEntry ExecutableAllocator::s_usageLog[256];
+int ExecutableAllocator::s_logMaxIndex = 0;
+CRITSEC_COOKIE ExecutableAllocator::s_LoggerCriticalSection;
+
+class StopWatch
+{
+    LARGE_INTEGER m_start;
+    int64_t* m_accumulator;
+
+public:
+    StopWatch(int64_t* accumulator) : m_accumulator(accumulator)
+    {
+        QueryPerformanceCounter(&m_start);
+    }
+
+    ~StopWatch()
+    {
+        LARGE_INTEGER end;
+        QueryPerformanceCounter(&end);
+
+        InterlockedExchangeAdd64(m_accumulator, end.QuadPart - m_start.QuadPart);
+    }
+};
+
+void ExecutableAllocator::LogUsage(const char* source, int line, const char* function)
+{
+    CRITSEC_Holder csh(s_LoggerCriticalSection);
+
+    for (int i = 0; i < s_logMaxIndex; i++)
+    {
+        if (s_usageLog[i].source == source && s_usageLog[i].line == line)
+        {
+            s_usageLog[i].count++;
+            return;
+        }
+    }
+
+    int i = s_logMaxIndex;
+    s_logMaxIndex++;
+    s_usageLog[i].source = source;
+    s_usageLog[i].function = function;
+    s_usageLog[i].line = line;
+    s_usageLog[i].count = 1;
+}
+
+void ExecutableAllocator::DumpHolderUsage()
+{
+    CRITSEC_Holder csh(s_LoggerCriticalSection);
+
+    LARGE_INTEGER freq;
+    QueryPerformanceFrequency(&freq);
+
+    fprintf(stderr, "Map time with lock sum: %I64dms\n", g_mapTimeWithLockSum / (freq.QuadPart / 1000));
+    fprintf(stderr, "Map time sum: %I64dms\n", g_mapTimeSum / (freq.QuadPart / 1000));
+    fprintf(stderr, "Map find RX time sum: %I64dms\n", g_mapFindRXTimeSum / (freq.QuadPart / 1000));
+    fprintf(stderr, "Map create time sum: %I64dms\n", g_mapCreateTimeSum / (freq.QuadPart / 1000));
+    fprintf(stderr, "Unmap time with lock sum: %I64dms\n", g_unmapTimeWithLockSum / (freq.QuadPart / 1000));
+    fprintf(stderr, "Unmap time sum: %I64dms\n", g_unmapTimeSum / (freq.QuadPart / 1000));
+
+    fprintf(stderr, "Reserve count: %I64d\n", g_reserveCount);
+    fprintf(stderr, "Release count: %I64d\n", g_releaseCount);
+
+    fprintf(stderr, "ExecutableWriterHolder usage:\n");
+
+    for (int i = 0; i < s_logMaxIndex; i++)
+    {
+        fprintf(stderr, "Count: %d at %s:%d in %s\n", s_usageLog[i].count, s_usageLog[i].source, s_usageLog[i].line, s_usageLog[i].function);
+    }
+}
+
+#endif // LOG_EXECUTABLE_ALLOCATOR_STATISTICS
+
 bool ExecutableAllocator::IsDoubleMappingEnabled()
 {
     LIMITED_METHOD_CONTRACT;
@@ -50,12 +132,9 @@ size_t ExecutableAllocator::Granularity()
     return g_SystemInfo.dwAllocationGranularity;
 }
 
-// Use this function to initialize the g_codeAllocHint
-// during startup. base is runtime .dll base address,
-// size is runtime .dll virtual size.
-void ExecutableAllocator::InitCodeAllocHint(size_t base, size_t size, int randomPageOffset)
+void ExecutableAllocator::InitLazyPreferredRange(size_t base, size_t size, int randomPageOffset)
 {
-#if USE_UPPER_ADDRESS
+#if USE_LAZY_PREFERRED_RANGE
 
 #ifdef _DEBUG
     // If GetForceRelocs is enabled we don't constrain the pMinAddr
@@ -64,39 +143,25 @@ void ExecutableAllocator::InitCodeAllocHint(size_t base, size_t size, int random
 #endif
 
     //
-    // If we are using the UPPER_ADDRESS space (on Win64)
-    // then for any code heap that doesn't specify an address
-    // range using [pMinAddr..pMaxAddr] we place it in the
-    // upper address space
-    // This enables us to avoid having to use long JumpStubs
-    // to reach the code for our ngen-ed images.
-    // Which are also placed in the UPPER_ADDRESS space.
+    // If we are using USE_LAZY_PREFERRED_RANGE then we try to allocate memory close
+    // to coreclr.dll.  This avoids having to create jump stubs for calls to
+    // helpers and R2R images loaded close to coreclr.dll.
     //
     SIZE_T reach = 0x7FFF0000u;
 
-    // We will choose the preferred code region based on the address of clr.dll. The JIT helpers
-    // in clr.dll are the most heavily called functions.
-    g_codeMinAddr = (base + size > reach) ? (BYTE *)(base + size - reach) : (BYTE *)0;
-    g_codeMaxAddr = (base + reach > base) ? (BYTE *)(base + reach) : (BYTE *)-1;
+    // We will choose the preferred code region based on the address of coreclr.dll. The JIT helpers
+    // in coreclr.dll are the most heavily called functions.
+    g_preferredRangeMin = (base + size > reach) ? (BYTE *)(base + size - reach) : (BYTE *)0;
+    g_preferredRangeMax = (base + reach > base) ? (BYTE *)(base + reach) : (BYTE *)-1;
 
     BYTE * pStart;
 
-    if (g_codeMinAddr <= (BYTE *)CODEHEAP_START_ADDRESS &&
-        (BYTE *)CODEHEAP_START_ADDRESS < g_codeMaxAddr)
-    {
-        // clr.dll got loaded at its preferred base address? (OS without ASLR - pre-Vista)
-        // Use the code head start address that does not cause collisions with NGen images.
-        // This logic is coupled with scripts that we use to assign base addresses.
-        pStart = (BYTE *)CODEHEAP_START_ADDRESS;
-    }
-    else
     if (base > UINT32_MAX)
     {
-        // clr.dll got address assigned by ASLR?
         // Try to occupy the space as far as possible to minimize collisions with other ASLR assigned
         // addresses. Do not start at g_codeMinAddr exactly so that we can also reach common native images
-        // that can be placed at higher addresses than clr.dll.
-        pStart = g_codeMinAddr + (g_codeMaxAddr - g_codeMinAddr) / 8;
+        // that can be placed at higher addresses than coreclr.dll.
+        pStart = g_preferredRangeMin + (g_preferredRangeMax - g_preferredRangeMin) / 8;
     }
     else
     {
@@ -108,31 +173,35 @@ void ExecutableAllocator::InitCodeAllocHint(size_t base, size_t size, int random
     // Randomize the address space
     pStart += GetOsPageSize() * randomPageOffset;
 
-    g_codeAllocStart = pStart;
-    g_codeAllocHint = pStart;
+    g_lazyPreferredRangeStart = pStart;
+    g_lazyPreferredRangeHint = pStart;
 #endif
 }
 
-// Use this function to reset the g_codeAllocHint
-// after unloading an AppDomain
-void ExecutableAllocator::ResetCodeAllocHint()
+void ExecutableAllocator::InitPreferredRange()
+{
+#ifdef TARGET_UNIX
+    void *start, *end;
+    PAL_GetExecutableMemoryAllocatorPreferredRange(&start, &end);
+    g_preferredRangeMin = (BYTE *)start;
+    g_preferredRangeMax = (BYTE *)end;
+#endif
+}
+
+void ExecutableAllocator::ResetLazyPreferredRangeHint()
 {
     LIMITED_METHOD_CONTRACT;
-#if USE_UPPER_ADDRESS
-    g_codeAllocHint = g_codeAllocStart;
+#if USE_LAZY_PREFERRED_RANGE
+    g_lazyPreferredRangeHint = g_lazyPreferredRangeStart;
 #endif
 }
-
-// Returns TRUE if p is located in near clr.dll that allows us
-// to use rel32 IP-relative addressing modes.
+// Returns TRUE if p is located in the memory area where we prefer to put
+// executable code and static fields. This area is typically close to the
+// coreclr library.
 bool ExecutableAllocator::IsPreferredExecutableRange(void * p)
 {
     LIMITED_METHOD_CONTRACT;
-#if USE_UPPER_ADDRESS
-    if (g_codeMinAddr <= (BYTE *)p && (BYTE *)p < g_codeMaxAddr)
-        return true;
-#endif
-    return false;
+    return g_preferredRangeMin <= (BYTE *)p && (BYTE *)p < g_preferredRangeMax;
 }
 
 ExecutableAllocator* ExecutableAllocator::Instance()
@@ -166,6 +235,9 @@ HRESULT ExecutableAllocator::StaticInitialize(FatalErrorHandler fatalErrorHandle
         return E_FAIL;
     }
 
+#ifdef LOG_EXECUTABLE_ALLOCATOR_STATISTICS
+    s_LoggerCriticalSection = ClrCreateCriticalSection(CrstExecutableAllocatorLock, CrstFlags(CRST_UNSAFE_ANYMODE | CRST_DEBUGGER_THREAD));
+#endif
     return S_OK;
 }
 
@@ -177,7 +249,8 @@ bool ExecutableAllocator::Initialize()
     {
         if (!VMToOSInterface::CreateDoubleMemoryMapper(&m_doubleMemoryMapperHandle, &m_maxExecutableCodeSize))
         {
-            return false;
+            g_isWXorXEnabled = false;
+            return true;
         }
 
         m_CriticalSection = ClrCreateCriticalSection(CrstExecutableAllocatorLock,CrstFlags(CRST_UNSAFE_ANYMODE | CRST_DEBUGGER_THREAD));
@@ -186,7 +259,7 @@ bool ExecutableAllocator::Initialize()
     return true;
 }
 
-//#define ENABLE_CACHED_MAPPINGS
+#define ENABLE_CACHED_MAPPINGS
 
 void ExecutableAllocator::UpdateCachedMapping(BlockRW* pBlock)
 {
@@ -213,7 +286,7 @@ void ExecutableAllocator::UpdateCachedMapping(BlockRW* pBlock)
         m_cachedMapping = pBlock;
         pBlock->refCount++;
     }
-#endif // ENABLE_CACHED_MAPPINGS    
+#endif // ENABLE_CACHED_MAPPINGS
 }
 
 void* ExecutableAllocator::FindRWBlock(void* baseRX, size_t size)
@@ -224,7 +297,11 @@ void* ExecutableAllocator::FindRWBlock(void* baseRX, size_t size)
     {
         if (pBlock->baseRX <= baseRX && ((size_t)baseRX + size) <= ((size_t)pBlock->baseRX + pBlock->size))
         {
-            pBlock->refCount++;
+#ifdef TARGET_64BIT
+            InterlockedIncrement64((LONG64*)& pBlock->refCount);
+#else
+            InterlockedIncrement((LONG*)&pBlock->refCount);
+#endif
             UpdateCachedMapping(pBlock);
 
             return (BYTE*)pBlock->baseRW + ((size_t)baseRX - (size_t)pBlock->baseRX);
@@ -238,14 +315,6 @@ bool ExecutableAllocator::AddRWBlock(void* baseRW, void* baseRX, size_t size)
 {
     LIMITED_METHOD_CONTRACT;
 
-    for (BlockRW* pBlock = m_pFirstBlockRW; pBlock != NULL; pBlock = pBlock->next)
-    {
-        if (pBlock->baseRX <= baseRX && ((size_t)baseRX + size) <= ((size_t)pBlock->baseRX + pBlock->size))
-        {
-            break;
-        }
-    }
-
     // The new "nothrow" below failure is handled as fail fast since it is not recoverable
     PERMANENT_CONTRACT_VIOLATION(FaultViolation, ReasonContractInfrastructure);
 
@@ -352,6 +421,10 @@ void ExecutableAllocator::Release(void* pRX)
 {
     LIMITED_METHOD_CONTRACT;
 
+#ifdef LOG_EXECUTABLE_ALLOCATOR_STATISTICS
+    InterlockedIncrement64(&g_releaseCount);
+#endif
+
     if (IsDoubleMappingEnabled())
     {
         CRITSEC_Holder csh(m_CriticalSection);
@@ -398,54 +471,40 @@ void ExecutableAllocator::Release(void* pRX)
     }
 }
 
-// Find a free block with the closest size >= the requested size.
+// Find a free block with the size == the requested size.
 // Returns NULL if no such block exists.
 ExecutableAllocator::BlockRX* ExecutableAllocator::FindBestFreeBlock(size_t size)
 {
     LIMITED_METHOD_CONTRACT;
 
     BlockRX* pPrevBlock = NULL;
-    BlockRX* pPrevBestBlock = NULL;
-    BlockRX* pBestBlock = NULL;
     BlockRX* pBlock = m_pFirstFreeBlockRX;
 
     while (pBlock != NULL)
     {
-        if (pBlock->size >= size)
+        if (pBlock->size == size)
         {
-            if (pBestBlock != NULL)
-            {
-                if (pBlock->size < pBestBlock->size)
-                {
-                    pPrevBestBlock = pPrevBlock;
-                    pBestBlock = pBlock;
-                }
-            }
-            else
-            {
-                pPrevBestBlock = pPrevBlock;
-                pBestBlock = pBlock;
-            }
+            break;
         }
         pPrevBlock = pBlock;
         pBlock = pBlock->next;
     }
 
-    if (pBestBlock != NULL)
+    if (pBlock != NULL)
     {
-        if (pPrevBestBlock != NULL)
+        if (pPrevBlock != NULL)
         {
-            pPrevBestBlock->next = pBestBlock->next;
+            pPrevBlock->next = pBlock->next;
         }
         else
         {
-            m_pFirstFreeBlockRX = pBestBlock->next;
+            m_pFirstFreeBlockRX = pBlock->next;
         }
 
-        pBestBlock->next = NULL;
+        pBlock->next = NULL;
     }
 
-    return pBestBlock;
+    return pBlock;
 }
 
 // Allocate a new block of executable memory and the related descriptor structure.
@@ -503,6 +562,10 @@ void* ExecutableAllocator::ReserveWithinRange(size_t size, const void* loAddress
 {
     LIMITED_METHOD_CONTRACT;
 
+#ifdef LOG_EXECUTABLE_ALLOCATOR_STATISTICS
+    InterlockedIncrement64(&g_reserveCount);
+#endif
+
     _ASSERTE((size & (Granularity() - 1)) == 0);
     if (IsDoubleMappingEnabled())
     {
@@ -522,7 +585,7 @@ void* ExecutableAllocator::ReserveWithinRange(size_t size, const void* loAddress
             block->baseRX = result;
             AddRXBlock(block);
         }
-        else 
+        else
         {
             BackoutBlock(block, isFreeBlock);
         }
@@ -549,11 +612,15 @@ void* ExecutableAllocator::Reserve(size_t size)
 {
     LIMITED_METHOD_CONTRACT;
 
+#ifdef LOG_EXECUTABLE_ALLOCATOR_STATISTICS
+    InterlockedIncrement64(&g_reserveCount);
+#endif
+
     _ASSERTE((size & (Granularity() - 1)) == 0);
 
     BYTE *result = NULL;
 
-#if USE_UPPER_ADDRESS
+#if USE_LAZY_PREFERRED_RANGE
     //
     // If we are using the UPPER_ADDRESS space (on Win64)
     // then for any heap that will contain executable code
@@ -563,32 +630,32 @@ void* ExecutableAllocator::Reserve(size_t size)
     // to reach the code for our ngen-ed images on x64,
     // since they are also placed in the UPPER_ADDRESS space.
     //
-    BYTE * pHint = g_codeAllocHint;
+    BYTE * pHint = g_lazyPreferredRangeHint;
 
-    if (size <= (SIZE_T)(g_codeMaxAddr - g_codeMinAddr) && pHint != NULL)
+    if (size <= (SIZE_T)(g_preferredRangeMax - g_preferredRangeMin) && pHint != NULL)
     {
         // Try to allocate in the preferred region after the hint
-        result = (BYTE*)ReserveWithinRange(size, pHint, g_codeMaxAddr);
+        result = (BYTE*)ReserveWithinRange(size, pHint, g_preferredRangeMax);
         if (result != NULL)
         {
-            g_codeAllocHint = result + size;
+            g_lazyPreferredRangeHint = result + size;
         }
         else
         {
             // Try to allocate in the preferred region before the hint
-            result = (BYTE*)ReserveWithinRange(size, g_codeMinAddr, pHint + size);
+            result = (BYTE*)ReserveWithinRange(size, g_preferredRangeMin, pHint + size);
 
             if (result != NULL)
             {
-                g_codeAllocHint = result + size;
+                g_lazyPreferredRangeHint = result + size;
             }
 
-            g_codeAllocHint = NULL;
+            g_lazyPreferredRangeHint = NULL;
         }
     }
 
     // Fall through to
-#endif // USE_UPPER_ADDRESS
+#endif // USE_LAZY_PREFERRED_RANGE
 
     if (result == NULL)
     {
@@ -610,7 +677,7 @@ void* ExecutableAllocator::Reserve(size_t size)
                 block->baseRX = result;
                 AddRXBlock(block);
             }
-            else 
+            else
             {
                 BackoutBlock(block, isFreeBlock);
             }
@@ -637,6 +704,10 @@ void* ExecutableAllocator::ReserveAt(void* baseAddressRX, size_t size)
 {
     LIMITED_METHOD_CONTRACT;
 
+#ifdef LOG_EXECUTABLE_ALLOCATOR_STATISTICS
+    InterlockedIncrement64(&g_reserveCount);
+#endif
+
     _ASSERTE((size & (Granularity() - 1)) == 0);
 
     if (IsDoubleMappingEnabled())
@@ -657,7 +728,7 @@ void* ExecutableAllocator::ReserveAt(void* baseAddressRX, size_t size)
             block->baseRX = result;
             AddRXBlock(block);
         }
-        else 
+        else
         {
             BackoutBlock(block, isFreeBlock);
         }
@@ -682,30 +753,45 @@ void* ExecutableAllocator::MapRW(void* pRX, size_t size)
         return pRX;
     }
 
+#ifdef LOG_EXECUTABLE_ALLOCATOR_STATISTICS
+    StopWatch swAll(&g_mapTimeWithLockSum);
+#endif
+
     CRITSEC_Holder csh(m_CriticalSection);
 
+#ifdef LOG_EXECUTABLE_ALLOCATOR_STATISTICS
+    StopWatch sw(&g_mapTimeSum);
+#endif
+
     void* result = FindRWBlock(pRX, size);
     if (result != NULL)
     {
         return result;
     }
+#ifdef LOG_EXECUTABLE_ALLOCATOR_STATISTICS
+    StopWatch sw2(&g_mapFindRXTimeSum);
+#endif
 
     for (BlockRX* pBlock = m_pFirstBlockRX; pBlock != NULL; pBlock = pBlock->next)
     {
         if (pRX >= pBlock->baseRX && ((size_t)pRX + size) <= ((size_t)pBlock->baseRX + pBlock->size))
         {
-            // Offset of the RX address in the originally allocated block
-            size_t offset = (size_t)pRX - (size_t)pBlock->baseRX;
-            // Offset of the RX address that will start the newly mapped block
-            size_t mapOffset = ALIGN_DOWN(offset, Granularity());
-            // Size of the block we will map
-            size_t mapSize = ALIGN_UP(offset - mapOffset + size, Granularity());
-            void* pRW = VMToOSInterface::GetRWMapping(m_doubleMemoryMapperHandle, (BYTE*)pBlock->baseRX + mapOffset, pBlock->offset + mapOffset, mapSize);
+        // Offset of the RX address in the originally allocated block
+        size_t offset = (size_t)pRX - (size_t)pBlock->baseRX;
+        // Offset of the RX address that will start the newly mapped block
+        size_t mapOffset = ALIGN_DOWN(offset, Granularity());
+        // Size of the block we will map
+        size_t mapSize = ALIGN_UP(offset - mapOffset + size, Granularity());
 
-            if (pRW == NULL)
-            {
-                g_fatalErrorHandler(COR_E_EXECUTIONENGINE, W("Failed to create RW mapping for RX memory"));
-            }
+#ifdef LOG_EXECUTABLE_ALLOCATOR_STATISTICS
+        StopWatch sw2(&g_mapCreateTimeSum);
+#endif
+        void* pRW = VMToOSInterface::GetRWMapping(m_doubleMemoryMapperHandle, (BYTE*)pBlock->baseRX + mapOffset, pBlock->offset + mapOffset, mapSize);
+
+        if (pRW == NULL)
+        {
+            g_fatalErrorHandler(COR_E_EXECUTIONENGINE, W("Failed to create RW mapping for RX memory"));
+        }
 
             AddRWBlock(pRW, (BYTE*)pBlock->baseRX + mapOffset, mapSize);
 
@@ -732,6 +818,10 @@ void ExecutableAllocator::UnmapRW(void* pRW)
 {
     LIMITED_METHOD_CONTRACT;
 
+#ifdef LOG_EXECUTABLE_ALLOCATOR_STATISTICS
+    StopWatch swAll(&g_unmapTimeWithLockSum);
+#endif
+
     if (!IsDoubleMappingEnabled())
     {
         return;
@@ -740,6 +830,10 @@ void ExecutableAllocator::UnmapRW(void* pRW)
     CRITSEC_Holder csh(m_CriticalSection);
     _ASSERTE(pRW != NULL);
 
+#ifdef LOG_EXECUTABLE_ALLOCATOR_STATISTICS
+    StopWatch swNoLock(&g_unmapTimeSum);
+#endif
+
     void* unmapAddress = NULL;
     size_t unmapSize;
 
