@@ -18,10 +18,6 @@
 #include "virtualcallstub.h"
 #include "decodemd.h"
 
-#ifdef FEATURE_PREJIT
-#include "compile.h"
-#endif
-
 #if defined(_DEBUG)
 DummyGlobalContract ___contract;
 #endif
@@ -44,14 +40,13 @@ MethodDesc * DispatchSlot::GetMethodDesc()
 }
 
 //------------------------------------------------------------------------
-void TypeIDMap::Init(UINT32 idStartValue, UINT32 idIncrementValue)
+void TypeIDMap::Init()
 {
     STANDARD_VM_CONTRACT;
 
     LockOwner lock = {&m_lock, IsOwnerOfCrst};
     m_idMap.Init(11, TRUE, &lock);
     m_mtMap.Init(11, TRUE, &lock);
-    m_idProvider.Init(idStartValue, idIncrementValue);
     m_entryCount = 0;
 }
 
@@ -67,7 +62,6 @@ UINT32 TypeIDMap::LookupTypeID(PTR_MethodTable pMT)
     } CONTRACTL_END;
 
     UINT32 id = (UINT32) m_mtMap.LookupValue((UPTR)dac_cast<TADDR>(pMT), 0);
-    _ASSERTE(!pMT->RequiresFatDispatchTokens() || (DispatchToken::RequiresDispatchTokenFat(id, 0)));
 
     return id;
 }
@@ -82,9 +76,6 @@ PTR_MethodTable TypeIDMap::LookupType(UINT32 id)
         PRECONDITION(id <= TypeIDProvider::MAX_TYPE_ID);
     } CONTRACTL_END;
 
-    if (!m_idProvider.OwnsID(id))
-        return NULL;
-
     UPTR ret = m_idMap.LookupValue((UPTR)id, 0);
     if (ret == static_cast<UPTR>(INVALIDENTRY))
         return NULL;
@@ -119,15 +110,7 @@ UINT32 TypeIDMap::GetTypeID(PTR_MethodTable pMT)
         {
             return id;
         }
-        // Get the next ID
-        if (pMT->RequiresFatDispatchTokens())
-        {
-            id = GetNextFatID();
-        }
-        else
-        {
-            id = GetNextID();
-        }
+        id = GetNextID();
 
         CONSISTENCY_CHECK(id <= TypeIDProvider::MAX_TYPE_ID);
         // Insert the pair, with lookups in both directions
@@ -135,8 +118,7 @@ UINT32 TypeIDMap::GetTypeID(PTR_MethodTable pMT)
         m_idMap.InsertValue((UPTR)id, (UPTR)pMT >> 1);
         m_mtMap.InsertValue((UPTR)pMT, (UPTR)id);
         m_entryCount++;
-        CONSISTENCY_CHECK(GetThread()->GetDomain()->IsCompilationDomain() ||
-                          (LookupType(id) == pMT));
+        CONSISTENCY_CHECK((LookupType(id) == pMT));
     }
 #else // DACCESS_COMPILE
     if (id == TypeIDProvider::INVALID_TYPE_ID)
@@ -487,37 +469,6 @@ DispatchMap::CreateEncodedMapping(
 #endif //_DEBUG
 } // DispatchMap::CreateEncodedMapping
 
-#ifdef FEATURE_NATIVE_IMAGE_GENERATION
-//------------------------------------------------------------------------
-void DispatchMap::Save(DataImage * image)
-{
-    STANDARD_VM_CONTRACT;
-
-    CONSISTENCY_CHECK(!image->IsStored(this));
-
-    UINT32 cbMap = GetMapSize();
-    UINT32 cbObj = GetObjectSize(cbMap);
-
-    image->StoreInternedStructure(
-        this,
-        cbObj,
-        DataImage::ITEM_DISPATCH_MAP,
-        sizeof(void *));
-
-#ifdef LOGGING
-    g_sdStats.m_cNGENDispatchMap++;
-    g_sdStats.m_cbNGENDispatchMap += cbObj;
-#endif //LOGGING
-}
-
-//------------------------------------------------------------------------
-void DispatchMap::Fixup(DataImage *image)
-{
-    STANDARD_VM_CONTRACT;
-}
-
-#endif //FEATURE_NATIVE_IMAGE_GENERATION
-
 #endif //!DACCESS_COMPILE
 
 //------------------------------------------------------------------------
