@@ -17,9 +17,8 @@
 #define INVALID_RESUME_ADDRESS 0x000000000000bad0
 
 EXTERN_C EXCEPTION_DISPOSITION
-ProcessCLRException(IN     PEXCEPTION_RECORD     pExceptionRecord
-          BIT64_ARG(IN     ULONG64               MemoryStackFp)
-      NOT_BIT64_ARG(IN     ULONG                 MemoryStackFp),
+ProcessCLRException(IN     PEXCEPTION_RECORD     pExceptionRecord,
+                    IN     PVOID                 pEstablisherFrame,
                     IN OUT PT_CONTEXT            pContextRecord,
                     IN OUT PT_DISPATCHER_CONTEXT pDispatcherContext);
 
@@ -360,7 +359,7 @@ public:
     void EnumMemoryRegions(CLRDataEnumMemoryFlags flags);
 #endif // DACCESS_COMPILE
 
-    static void DebugLogTrackerRanges(__in_z const char *pszTag);
+    static void DebugLogTrackerRanges(_In_z_ const char *pszTag);
 
     bool IsStackOverflowException();
 
@@ -380,11 +379,8 @@ private:
                     StackFrame              sf,
                     EE_ILEXCEPTION_CLAUSE*  pEHClause,
                     MethodDesc*             pMD,
-                    EHFuncletType funcletType
-                    X86_ARG(PT_CONTEXT pContextRecord)
-                    ARM_ARG(PT_CONTEXT pContextRecord)
-                    ARM64_ARG(PT_CONTEXT pContextRecord)
-                    );
+                    EHFuncletType           funcletType,
+                    PT_CONTEXT              pContextRecord);
 
     inline static BOOL
         ClauseCoversPC(EE_ILEXCEPTION_CLAUSE* pEHClause,
