@@ -420,7 +420,6 @@ COR_IL_MAP* ProfilerFunctionControl::GetInstrumentedMapEntries()
 #ifndef DACCESS_COMPILE
 NativeImageInliningIterator::NativeImageInliningIterator() :
         m_pModule(NULL),
-        m_pInlinee(NULL),
         m_dynamicBuffer(NULL),
         m_dynamicBufferSize(0),
         m_dynamicAvailable(0),
@@ -429,22 +428,22 @@ NativeImageInliningIterator::NativeImageInliningIterator() :
 
 }
 
-HRESULT NativeImageInliningIterator::Reset(Module *pModule, MethodDesc *pInlinee)
+HRESULT NativeImageInliningIterator::Reset(Module *pInlinerModule, MethodInModule inlinee)
 {
-    _ASSERTE(pModule != NULL);
-    _ASSERTE(pInlinee != NULL);
+    _ASSERTE(pInlinerModule != NULL);
+    _ASSERTE(inlinee.m_module != NULL);
 
-    m_pModule = pModule;
-    m_pInlinee = pInlinee;
+    m_pModule = pInlinerModule;
+    m_inlinee = inlinee;
 
     HRESULT hr = S_OK;
     EX_TRY
     {
         // Trying to use the existing buffer
         BOOL incompleteData;
-        Module *inlineeModule = m_pInlinee->GetModule();
-        mdMethodDef mdInlinee = m_pInlinee->GetMemberDef();
-        COUNT_T methodsAvailable = m_pModule->GetNativeOrReadyToRunInliners(inlineeModule, mdInlinee, m_dynamicBufferSize, m_dynamicBuffer, &incompleteData);
+        Module *inlineeModule = m_inlinee.m_module;
+        mdMethodDef mdInlinee = m_inlinee.m_methodDef;
+        COUNT_T methodsAvailable = m_pModule->GetReadyToRunInliners(inlineeModule, mdInlinee, m_dynamicBufferSize, m_dynamicBuffer, &incompleteData);
 
         // If the existing buffer is not large enough, reallocate.
         if (methodsAvailable > m_dynamicBufferSize)
@@ -453,7 +452,7 @@ HRESULT NativeImageInliningIterator::Reset(Module *pModule, MethodDesc *pInlinee
             m_dynamicBuffer = new MethodInModule[newSize];
             m_dynamicBufferSize = newSize;
 
-            methodsAvailable = m_pModule->GetNativeOrReadyToRunInliners(inlineeModule, mdInlinee, m_dynamicBufferSize, m_dynamicBuffer, &incompleteData);
+            methodsAvailable = m_pModule->GetReadyToRunInliners(inlineeModule, mdInlinee, m_dynamicBufferSize, m_dynamicBuffer, &incompleteData);
             _ASSERTE(methodsAvailable <= m_dynamicBufferSize);
         }
 
@@ -484,19 +483,16 @@ BOOL NativeImageInliningIterator::Next()
     return m_currentPos < m_dynamicAvailable;
 }
 
-MethodDesc *NativeImageInliningIterator::GetMethodDesc()
+MethodInModule NativeImageInliningIterator::GetMethod()
 {
     // this evaluates true when m_currentPos == s_failurePos or m_currentPos == (COUNT_T)-1
     // m_currentPos is an unsigned type
     if (m_currentPos >= m_dynamicAvailable)
     {
-        return NULL;
+        return MethodInModule();
     }
 
-    MethodInModule mm = m_dynamicBuffer[m_currentPos];
-    Module *pModule = mm.m_module;
-    mdMethodDef mdInliner = mm.m_methodDef;
-    return pModule->LookupMethodDef(mdInliner);
+    return m_dynamicBuffer[m_currentPos];
 }
 
 //---------------------------------------------------------------------------------------
@@ -623,16 +619,20 @@ HRESULT ReJitManager::UpdateActiveILVersions(
 
         if ((flags & COR_PRF_REJIT_BLOCK_INLINING) == COR_PRF_REJIT_BLOCK_INLINING)
         {
-            hr = UpdateNativeInlinerActiveILVersions(&mgrToCodeActivationBatch, pMD, fIsRevert, flags);
+            hr = UpdateNativeInlinerActiveILVersions(&mgrToCodeActivationBatch, pModule, rgMethodDefs[i], fIsRevert, flags);
             if (FAILED(hr))
             {
                 return hr;
             }
 
-            hr = UpdateJitInlinerActiveILVersions(&mgrToCodeActivationBatch, pMD, fIsRevert, flags);
-            if (FAILED(hr))
+            if (pMD != NULL)
             {
-                return hr;
+                // If pMD is not null, then the method may have already been inlined somewhere. Go check.
+                hr = UpdateJitInlinerActiveILVersions(&mgrToCodeActivationBatch, pMD, fIsRevert, flags);
+                if (FAILED(hr))
+                {
+                    return hr;
+                }
             }
         }
     }   // for (ULONG i = 0; i < cFunctions; i++)
@@ -643,8 +643,6 @@ HRESULT ReJitManager::UpdateActiveILVersions(
     SHash<CodeActivationBatchTraits>::Iterator endIter = mgrToCodeActivationBatch.End();
 
     {
-        MethodDescBackpatchInfoTracker::ConditionalLockHolderForGCCoop slotBackpatchLockHolder;
-
         for (SHash<CodeActivationBatchTraits>::Iterator iter = beginIter; iter != endIter; iter++)
         {
             CodeActivationBatch * pCodeActivationBatch = *iter;
@@ -782,7 +780,8 @@ HRESULT ReJitManager::UpdateActiveILVersion(
 // static
 HRESULT ReJitManager::UpdateNativeInlinerActiveILVersions(
     SHash<CodeActivationBatchTraits>   *pMgrToCodeActivationBatch,
-    MethodDesc                         *pInlinee,
+    Module                             *pInlineeModule,
+    mdMethodDef                         inlineeMethodDef,
     BOOL                                fIsRevert,
     COR_PRF_REJIT_FLAGS                 flags)
 {
@@ -796,7 +795,8 @@ HRESULT ReJitManager::UpdateNativeInlinerActiveILVersions(
     CONTRACTL_END;
 
     _ASSERTE(pMgrToCodeActivationBatch != NULL);
-    _ASSERTE(pInlinee != NULL);
+    _ASSERTE(pInlineeModule != NULL);
+    _ASSERTE(RidFromToken(inlineeMethodDef) != 0);
 
     HRESULT hr = S_OK;
 
@@ -811,35 +811,30 @@ HRESULT ReJitManager::UpdateNativeInlinerActiveILVersions(
         _ASSERTE(pDomainAssembly != NULL);
         _ASSERTE(pDomainAssembly->GetAssembly() != NULL);
 
-        DomainModuleIterator domainModuleIterator = pDomainAssembly->IterateModules(kModIterIncludeLoaded);
-        while (domainModuleIterator.Next())
+        Module * pModule = pDomainAssembly->GetModule();
+        if (pModule->HasReadyToRunInlineTrackingMap())
         {
-            Module * pCurModule = domainModuleIterator.GetModule();
-            if (pCurModule->HasNativeOrReadyToRunInlineTrackingMap())
+            inlinerIter.Reset(pModule, MethodInModule(pInlineeModule, inlineeMethodDef));
+
+            while (inlinerIter.Next())
             {
-                inlinerIter.Reset(pCurModule, pInlinee);
-
-                MethodDesc *pInliner = NULL;
-                while (inlinerIter.Next())
+                MethodInModule inliner = inlinerIter.GetMethod();
                 {
-                    pInliner = inlinerIter.GetMethodDesc();
+                    CodeVersionManager *pCodeVersionManager = pModule->GetCodeVersionManager();
+                    CodeVersionManager::LockHolder codeVersioningLockHolder;
+                    ILCodeVersion ilVersion = pCodeVersionManager->GetActiveILCodeVersion(inliner.m_module, inliner.m_methodDef);
+                    if (!ilVersion.HasDefaultIL())
                     {
-                        CodeVersionManager *pCodeVersionManager = pCurModule->GetCodeVersionManager();
-                        CodeVersionManager::LockHolder codeVersioningLockHolder;
-                        ILCodeVersion ilVersion = pCodeVersionManager->GetActiveILCodeVersion(pInliner);
-                        if (!ilVersion.HasDefaultIL())
-                        {
-                            // This method has already been ReJITted, no need to request another ReJIT at this point.
-                            // The ReJITted method will be in the JIT inliner check below.
-                            continue;
-                        }
+                        // This method has already been ReJITted, no need to request another ReJIT at this point.
+                        // The ReJITted method will be in the JIT inliner check below.
+                        continue;
                     }
+                }
 
-                    hr = UpdateActiveILVersion(pMgrToCodeActivationBatch, pInliner->GetModule(), pInliner->GetMemberDef(), fIsRevert, flags);
-                    if (FAILED(hr))
-                    {
-                        ReportReJITError(pInliner->GetModule(), pInliner->GetMemberDef(), NULL, hr);
-                    }
+                hr = UpdateActiveILVersion(pMgrToCodeActivationBatch, inliner.m_module, inliner.m_methodDef, fIsRevert, flags);
+                if (FAILED(hr))
+                {
+                    ReportReJITError(inliner.m_module, inliner.m_methodDef, NULL, hr);
                 }
             }
         }
@@ -1113,7 +1108,7 @@ HRESULT ReJitManager::ConfigureILCodeVersion(ILCodeVersion ilCodeVersion)
 
             if (FAILED(hr))
             {
-                // Only call if the GetReJITParamters call failed
+                // Only call if the GetReJITParameters call failed
                 ReportReJITError(pModule, methodDef, pModule->LookupMethodDef(methodDef), hr);
             }
             return S_OK;
@@ -1139,7 +1134,7 @@ HRESULT ReJitManager::ConfigureILCodeVersion(ILCodeVersion ilCodeVersion)
     }
     else if (fWaitForParameters)
     {
-        // This feels annoying, but it doesn't appear like we have the good threading primitves
+        // This feels annoying, but it doesn't appear like we have the good threading primitives
         // for this. What I would like is an AutoResetEvent that atomically exits the table
         // Crst when I wait on it. From what I can tell our AutoResetEvent doesn't have
         // that atomic transition which means this ordering could occur:
@@ -1166,7 +1161,7 @@ HRESULT ReJitManager::ConfigureILCodeVersion(ILCodeVersion ilCodeVersion)
                 CodeVersionManager::LockHolder codeVersioningLockHolder;
                 if (ilCodeVersion.GetRejitState() == ILCodeVersion::kStateActive)
                 {
-                    break; // the other thread got the parameters succesfully, go race to rejit
+                    break; // the other thread got the parameters successfully, go race to rejit
                 }
             }
             ClrSleepEx(1, FALSE);
