@@ -181,7 +181,7 @@ void TraceDestination::InitForUnjittedMethod(MethodDesc * pDesc)
             {
                 pDesc = pNewDesc;
 
-                LOG((LF_CORDB, LL_INFO10000, "TD::UnjittedMethod: wrapper md: %p --> %p", pDesc, pNewDesc));
+                LOG((LF_CORDB, LL_INFO10000, "TD::UnjittedMethod: wrapper md: %p --> %p\n", pDesc, pNewDesc));
 
             }
         }
@@ -635,7 +635,7 @@ void StubManager::AddStubManager(StubManager *mgr)
         g_pFirstManager = mgr;
     }
 
-    LOG((LF_CORDB, LL_EVERYTHING, "StubManager::AddStubManager - 0x%p (vptr %x%p)\n", mgr, (*(PVOID*)mgr)));
+    LOG((LF_CORDB, LL_EVERYTHING, "StubManager::AddStubManager - 0x%p (vptr %p)\n", mgr, (*(PVOID*)mgr)));
 }
 
 //-----------------------------------------------------------
@@ -675,8 +675,7 @@ StubManager::EnumMemoryRegions(CLRDataEnumMemoryFlags flags)
 {
     SUPPORTS_DAC;
     // Report the global list head.
-    DacEnumMemoryRegion(DacGlobalBase() +
-                        g_dacGlobals.StubManager__g_pFirstManager,
+    DacEnumMemoryRegion(DacGlobalValues()->StubManager__g_pFirstManager,
                         sizeof(TADDR));
 
     //
@@ -1004,8 +1003,7 @@ BOOL PrecodeStubManager::CheckIsStub_Internal(PCODE stubStartAddress)
     }
     CONTRACTL_END;
 
-    // Forwarded to from RangeSectionStubManager
-    return FALSE;
+    return GetStubPrecodeRangeList()->IsInRange(stubStartAddress) || GetFixupPrecodeRangeList()->IsInRange(stubStartAddress);
 }
 
 BOOL PrecodeStubManager::DoTraceStub(PCODE stubStartAddress,
@@ -1033,7 +1031,14 @@ BOOL PrecodeStubManager::DoTraceStub(PCODE stubStartAddress,
     else
 #endif // HAS_COMPACT_ENTRYPOINTS
     {
-        Precode* pPrecode = Precode::GetPrecodeFromEntryPoint(stubStartAddress);
+        // When the target slot points to the fixup part of the fixup precode, we need to compensate
+        // for that to get the actual stub address
+        Precode* pPrecode = Precode::GetPrecodeFromEntryPoint(stubStartAddress - FixupPrecode::FixupCodeOffset, TRUE /* speculative */);
+        if ((pPrecode == NULL) || (pPrecode->GetType() != PRECODE_FIXUP))
+        {
+            pPrecode = Precode::GetPrecodeFromEntryPoint(stubStartAddress);
+        }
+
         PREFIX_ASSUME(pPrecode != NULL);
 
         switch (pPrecode->GetType())
@@ -1169,13 +1174,13 @@ BOOL StubLinkStubManager::DoTraceStub(PCODE stubStartAddress,
     CONTRACTL_END
 
     LOG((LF_CORDB, LL_INFO10000,
-         "StubLinkStubManager::DoTraceStub: stubStartAddress=0x%08x\n",
+         "StubLinkStubManager::DoTraceStub: stubStartAddress=0x%p\n",
          stubStartAddress));
 
     Stub *stub = Stub::RecoverStub(stubStartAddress);
 
     LOG((LF_CORDB, LL_INFO10000,
-         "StubLinkStubManager::DoTraceStub: stub=0x%08x\n", stub));
+         "StubLinkStubManager::DoTraceStub: stub=0x%p\n", stub));
 
     //
     // If this is an intercept stub, we may be able to step
@@ -1188,39 +1193,60 @@ BOOL StubLinkStubManager::DoTraceStub(PCODE stubStartAddress,
     TADDR pRealAddr = 0;
     if (stub->IsMulticastDelegate())
     {
-        LOG((LF_CORDB, LL_INFO10000,
-             "StubLinkStubManager(MCDel)::DoTraceStub: stubStartAddress=0x%08x\n",
-             stubStartAddress));
-
-        LOG((LF_CORDB, LL_INFO10000,
-             "StubLinkStubManager(MCDel)::DoTraceStub: stub=0x%08x MGR_PUSH to entrypoint:0x%x\n", stub,
-             stub->GetEntryPoint()));
-
         // If it's a MC delegate, then we want to set a BP & do a context-ful
         // manager push, so that we can figure out if this call will be to a
         // single multicast delegate or a multi multicast delegate
         trace->InitForManagerPush(stubStartAddress, this);
-
+        LOG_TRACE_DESTINATION(trace, stubStartAddress, "StubLinkStubManager(MCDel)::DoTraceStub");
         return TRUE;
     }
-    else if (stub->GetPatchOffset() == 0)
+    else if (stub->IsInstantiatingStub())
     {
-        LOG((LF_CORDB, LL_INFO10000, "StubLinkStubManager::DoTraceStub: patch offset is 0!\n"));
-
-        return FALSE;
+        trace->InitForManagerPush(stubStartAddress, this);
+        LOG_TRACE_DESTINATION(trace, stubStartAddress, "StubLinkStubManager(InstantiatingMethod)::DoTraceStub");
+        return TRUE;
     }
-    else
+    else if (stub->GetPatchOffset() != 0)
     {
+        // The patch offset is currently only non-zero in x86 non-IL delegate scenarios.
         trace->InitForFramePush((PCODE)stub->GetPatchAddress());
-
         LOG_TRACE_DESTINATION(trace, stubStartAddress, "StubLinkStubManager::DoTraceStub");
-
         return TRUE;
     }
+
+    LOG((LF_CORDB, LL_INFO10000, "StubLinkStubManager::DoTraceStub: patch offset is 0!\n"));
+    return FALSE;
 }
 
 #ifndef DACCESS_COMPILE
 
+static PCODE GetStubTarget(PTR_MethodDesc pTargetMD)
+{
+    CONTRACTL
+    {
+        THROWS;
+        GC_TRIGGERS;
+        MODE_COOPERATIVE;
+        PRECONDITION(pTargetMD != NULL);
+    }
+    CONTRACTL_END;
+
+    NativeCodeVersion targetCode;
+
+#ifdef FEATURE_CODE_VERSIONING
+    CodeVersionManager::LockHolder codeVersioningLockHolder;
+    ILCodeVersion ilcode = pTargetMD->GetCodeVersionManager()->GetActiveILCodeVersion(pTargetMD);
+    targetCode = ilcode.GetActiveNativeCodeVersion(pTargetMD);
+#else
+    targetCode = NativeCodeVersion(pTargetMD);
+#endif
+
+    if (targetCode.IsNull() || targetCode.GetNativeCode() == NULL)
+        return NULL;
+
+    return targetCode.GetNativeCode();
+}
+
 BOOL StubLinkStubManager::TraceManager(Thread *thread,
                                        TraceDestination *trace,
                                        T_CONTEXT *pContext,
@@ -1236,21 +1262,39 @@ BOOL StubLinkStubManager::TraceManager(Thread *thread,
     }
     CONTRACTL_END
 
-    // NOTE that we're assuming that this will be called if and ONLY if
-    // we're examing a multicast delegate stub.  Otherwise, we'll have to figure out
-    // what we're looking iat
-
-    BYTE *pbDel = 0;
-
     LPVOID pc = (LPVOID)GetIP(pContext);
-
     *pRetAddr = (BYTE *)StubManagerHelpers::GetReturnAddress(pContext);
+    LOG((LF_CORDB,LL_INFO10000, "SLSM:TM 0x%p, retAddr is 0x%p\n", pc, (*pRetAddr)));
 
-    pbDel = (BYTE *)StubManagerHelpers::GetThisPtr(pContext);
+    Stub *stub = Stub::RecoverStub((PCODE)pc);
+    if (stub->IsInstantiatingStub())
+    {
+        LOG((LF_CORDB,LL_INFO10000, "SLSM:TM Instantiating method stub\n"));
+        PTR_MethodDesc pMD = stub->GetInstantiatedMethodDesc();
+        _ASSERTE(pMD != NULL);
 
-    LOG((LF_CORDB,LL_INFO10000, "SLSM:TM at 0x%x, retAddr is 0x%x\n", pc, (*pRetAddr)));
+        PCODE target = GetStubTarget(pMD);
+        if (target == NULL)
+        {
+            LOG((LF_CORDB,LL_INFO10000, "SLSM:TM Unable to determine stub target, fd 0x%p\n", pMD));
+            trace->InitForUnjittedMethod(pMD);
+            return TRUE;
+        }
 
-    return DelegateInvokeStubManager::TraceDelegateObject(pbDel, trace);
+        trace->InitForManaged(target);
+        return TRUE;
+    }
+    else if (stub->IsMulticastDelegate())
+    {
+        LOG((LF_CORDB,LL_INFO10000, "SLSM:TM MultiCastDelegate\n"));
+        BYTE *pbDel = (BYTE *)StubManagerHelpers::GetThisPtr(pContext);
+        return DelegateInvokeStubManager::TraceDelegateObject(pbDel, trace);
+    }
+
+    // Runtime bug if we get here. Did we make a change in StubLinkStubManager::DoTraceStub() that
+    // dispatched new stubs to TraceManager without writing the code to handle them?
+    _ASSERTE(!"SLSM:TM wasn't expected to handle any other stub types");
+    return FALSE;
 }
 
 #endif // #ifndef DACCESS_COMPILE
@@ -1420,40 +1464,6 @@ BOOL RangeSectionStubManager::DoTraceStub(PCODE stubStartAddress, TraceDestinati
     case STUB_CODE_BLOCK_STUBLINK:
         return StubLinkStubManager::g_pManager->DoTraceStub(stubStartAddress, trace);
 
-#ifdef FEATURE_PREJIT
-    case STUB_CODE_BLOCK_VIRTUAL_METHOD_THUNK:
-        {
-            PCODE pTarget = GetMethodThunkTarget(stubStartAddress);
-            if (pTarget == ExecutionManager::FindZapModule(stubStartAddress)->
-                                        GetNGenLayoutInfo()->m_pVirtualImportFixupJumpStub)
-            {
-#ifdef DACCESS_COMPILE
-                DacNotImpl();
-#else
-                trace->InitForManagerPush(GetEEFuncEntryPoint(VirtualMethodFixupPatchLabel), this);
-#endif
-            }
-            else
-            {
-                trace->InitForStub(pTarget);
-            }
-            return TRUE;
-        }
-
-    case STUB_CODE_BLOCK_EXTERNAL_METHOD_THUNK:
-        {
-            PCODE pTarget = GetMethodThunkTarget(stubStartAddress);
-            if (pTarget != ExecutionManager::FindZapModule(stubStartAddress)->
-                                        GetNGenLayoutInfo()->m_pExternalMethodFixupJumpStub)
-            {
-                trace->InitForStub(pTarget);
-                return TRUE;
-            }
-        }
-
-        FALLTHROUGH;
-#endif
-
     case STUB_CODE_BLOCK_METHOD_CALL_THUNK:
 #ifdef DACCESS_COMPILE
         DacNotImpl();
@@ -1483,14 +1493,7 @@ BOOL RangeSectionStubManager::TraceManager(Thread *thread,
     }
     CONTRACTL_END;
 
-#ifdef FEATURE_PREJIT
-    // Both virtual and external import thunks have the same structure. We can use
-    // common code to handle them.
-    _ASSERTE(GetIP(pContext) == GetEEFuncEntryPoint(VirtualMethodFixupPatchLabel)
-         || GetIP(pContext) == GetEEFuncEntryPoint(ExternalMethodFixupPatchLabel));
-#else
     _ASSERTE(GetIP(pContext) == GetEEFuncEntryPoint(ExternalMethodFixupPatchLabel));
-#endif
 
     *pRetAddr = (BYTE *)StubManagerHelpers::GetReturnAddress(pContext);
 
@@ -1500,21 +1503,6 @@ BOOL RangeSectionStubManager::TraceManager(Thread *thread,
 }
 #endif
 
-PCODE RangeSectionStubManager::GetMethodThunkTarget(PCODE stubStartAddress)
-{
-    WRAPPER_NO_CONTRACT;
-
-#if defined(TARGET_X86) || defined(TARGET_AMD64)
-    return rel32Decode(stubStartAddress+1);
-#elif defined(TARGET_ARM)
-    TADDR pInstr = PCODEToPINSTR(stubStartAddress);
-    return *dac_cast<PTR_PCODE>(pInstr + 2 * sizeof(DWORD));
-#else
-    PORTABILITY_ASSERT("RangeSectionStubManager::GetMethodThunkTarget");
-    return NULL;
-#endif
-}
-
 #ifdef DACCESS_COMPILE
 LPCWSTR RangeSectionStubManager::GetStubManagerName(PCODE addr)
 {
@@ -1636,7 +1624,7 @@ BOOL ILStubManager::DoTraceStub(PCODE stubStartAddress,
 
 #ifndef DACCESS_COMPILE
 #ifdef FEATURE_COMINTEROP
-PCODE ILStubManager::GetCOMTarget(Object *pThis, ComPlusCallInfo *pComPlusCallInfo)
+static PCODE GetCOMTarget(Object *pThis, ComPlusCallInfo *pComPlusCallInfo)
 {
     CONTRACTL
     {
@@ -1661,7 +1649,6 @@ PCODE ILStubManager::GetCOMTarget(Object *pThis, ComPlusCallInfo *pComPlusCallIn
 }
 #endif // FEATURE_COMINTEROP
 
-#ifndef CROSSGEN_COMPILE
 BOOL ILStubManager::TraceManager(Thread *thread,
                                  TraceDestination *trace,
                                  T_CONTEXT *pContext,
@@ -1681,27 +1668,24 @@ BOOL ILStubManager::TraceManager(Thread *thread,
 #endif
 
     DynamicMethodDesc *pStubMD = Entry2MethodDesc(stubIP, NULL)->AsDynamicMethodDesc();
-
     TADDR arg = StubManagerHelpers::GetHiddenArg(pContext);
-
     Object * pThis = StubManagerHelpers::GetThisPtr(pContext);
+    LOG((LF_CORDB, LL_INFO1000, "ILSM::TraceManager: Enter: StubMD 0x%p, HiddenArg 0x%p, ThisPtr 0x%p\n",
+        pStubMD, arg, pThis));
 
     // See code:ILStubCache.CreateNewMethodDesc for the code that sets flags on stub MDs
-    PCODE target;
+    PCODE target = NULL;
 
 #ifdef FEATURE_MULTICASTSTUB_AS_IL
-    if(pStubMD->IsMulticastStub())
+    if (pStubMD->IsMulticastStub())
     {
         _ASSERTE(GetIP(pContext) == GetEEFuncEntryPoint(StubHelpers::MulticastDebuggerTraceHelper));
 
         int delegateCount = (int)StubManagerHelpers::GetSecondArg(pContext);
-
         int totalDelegateCount = (int)*(size_t*)((BYTE*)pThis + DelegateObject::GetOffsetOfInvocationCount());
-
         if (delegateCount == totalDelegateCount)
         {
-            LOG((LF_CORDB, LL_INFO1000, "MF::TF: Executed all stubs, should return\n"));
-            // We've executed all the stubs, so we should return
+            LOG((LF_CORDB, LL_INFO1000, "ILSM::TraceManager: Fired all delegates\n"));
             return FALSE;
         }
         else
@@ -1715,7 +1699,6 @@ BOOL ILStubManager::TraceManager(Thread *thread,
             _ASSERTE(pbDel);
             return DelegateInvokeStubManager::TraceDelegateObject(pbDel, trace);
         }
-
     }
     else
 #endif // FEATURE_MULTICASTSTUB_AS_IL
@@ -1726,27 +1709,26 @@ BOOL ILStubManager::TraceManager(Thread *thread,
             // This is reverse P/Invoke stub, the argument is UMEntryThunk
             UMEntryThunk *pEntryThunk = (UMEntryThunk *)arg;
             target = pEntryThunk->GetManagedTarget();
-
-            LOG((LF_CORDB, LL_INFO10000, "ILSM::TraceManager: Reverse P/Invoke case 0x%x\n", target));
+            LOG((LF_CORDB, LL_INFO10000, "ILSM::TraceManager: Reverse P/Invoke case 0x%p\n", target));
         }
         else
         {
             // This is COM-to-CLR stub, the argument is the target
             target = (PCODE)arg;
-            LOG((LF_CORDB, LL_INFO10000, "ILSM::TraceManager: COM-to-CLR case 0x%x\n", target));
+            LOG((LF_CORDB, LL_INFO10000, "ILSM::TraceManager: COM-to-CLR case 0x%p\n", target));
         }
         trace->InitForManaged(target);
     }
-    else if (pStubMD->IsDelegateStub())
+    else if (pStubMD->HasFlags(DynamicMethodDesc::FlagIsDelegate))
     {
         // This is forward delegate P/Invoke stub, the argument is undefined
         DelegateObject *pDel = (DelegateObject *)pThis;
         target = pDel->GetMethodPtrAux();
 
-        LOG((LF_CORDB, LL_INFO10000, "ILSM::TraceManager: Forward delegate P/Invoke case 0x%x\n", target));
+        LOG((LF_CORDB, LL_INFO10000, "ILSM::TraceManager: Forward delegate P/Invoke case 0x%p\n", target));
         trace->InitForUnmanaged(target);
     }
-    else if (pStubMD->IsCALLIStub())
+    else if (pStubMD->HasFlags(DynamicMethodDesc::FlagIsCALLI))
     {
         // This is unmanaged CALLI stub, the argument is the target
         target = (PCODE)arg;
@@ -1756,29 +1738,41 @@ BOOL ILStubManager::TraceManager(Thread *thread,
         target = target >> 1; // call target is encoded as (addr << 1) | 1
 #endif // TARGET_AMD64
 
-        LOG((LF_CORDB, LL_INFO10000, "ILSM::TraceManager: Unmanaged CALLI case 0x%x\n", target));
+        LOG((LF_CORDB, LL_INFO10000, "ILSM::TraceManager: Unmanaged CALLI case 0x%p\n", target));
         trace->InitForUnmanaged(target);
     }
-    else if (pStubMD->IsStructMarshalStub())
+    else if (pStubMD->IsStepThroughStub())
     {
-        // There's no "target" for struct marshalling stubs
-        // so we have nowhere to tell the debugger to move the breakpoint.
-        return FALSE;
+        MethodDesc* pTargetMD = pStubMD->GetILStubResolver()->GetStubTargetMethodDesc();
+        if (pTargetMD == NULL)
+        {
+            LOG((LF_CORDB, LL_INFO1000, "ILSM::TraceManager: Stub has no target to step through to\n"));
+            return FALSE;
+        }
+
+        LOG((LF_CORDB, LL_INFO1000, "ILSM::TraceManager: Step through to target - 0x%p\n", pTargetMD));
+        target = GetStubTarget(pTargetMD);
+        if (target == NULL)
+            return FALSE;
+
+        trace->InitForManaged(target);
     }
-    else
+    else if (pStubMD->HasMDContextArg())
     {
+        LOG((LF_CORDB, LL_INFO1000, "ILSM::TraceManager: Hidden argument is MethodDesc\n"));
+
         // This is either direct forward P/Invoke or a CLR-to-COM call, the argument is MD
         MethodDesc *pMD = (MethodDesc *)arg;
-
         if (pMD->IsNDirect())
         {
             target = (PCODE)((NDirectMethodDesc *)pMD)->GetNativeNDirectTarget();
-            LOG((LF_CORDB, LL_INFO10000, "ILSM::TraceManager: Forward P/Invoke case 0x%x\n", target));
+            LOG((LF_CORDB, LL_INFO10000, "ILSM::TraceManager: Forward P/Invoke case 0x%p\n", target));
             trace->InitForUnmanaged(target);
         }
 #ifdef FEATURE_COMINTEROP
         else
         {
+            LOG((LF_CORDB, LL_INFO1000, "ILSM::TraceManager: Stub is CLR-to-COM\n"));
             _ASSERTE(pMD->IsComPlusCall());
             ComPlusCallMethodDesc *pCMD = (ComPlusCallMethodDesc *)pMD;
             _ASSERTE(!pCMD->IsStatic() && !pCMD->IsCtor() && "Static methods and constructors are not supported for built-in classic COM");
@@ -1786,17 +1780,21 @@ BOOL ILStubManager::TraceManager(Thread *thread,
             if (pThis != NULL)
             {
                 target = GetCOMTarget(pThis, pCMD->m_pComPlusCallInfo);
-
-                LOG((LF_CORDB, LL_INFO10000, "ILSM::TraceManager: CLR-to-COM case 0x%x\n", target));
+                LOG((LF_CORDB, LL_INFO10000, "ILSM::TraceManager: CLR-to-COM case 0x%p\n", target));
                 trace->InitForUnmanaged(target);
             }
         }
 #endif // FEATURE_COMINTEROP
     }
+    else
+    {
+        LOG((LF_CORDB, LL_INFO1000, "ILSM::TraceManager: No known target, IL Stub is a leaf\n"));
+        // There's no "target" so we have nowhere to tell the debugger to move the breakpoint.
+        return FALSE;
+    }
 
     return TRUE;
 }
-#endif // !CROSSGEN_COMPILE
 #endif //!DACCESS_COMPILE
 
 // This is used to recognize GenericComPlusCallStub, VarargPInvokeStub, and GenericPInvokeCalliHelper.
@@ -1829,7 +1827,7 @@ static BOOL IsVarargPInvokeStub(PCODE stubStartAddress)
     if (stubStartAddress == GetEEFuncEntryPoint(VarargPInvokeStub))
         return TRUE;
 
-#if !defined(TARGET_X86) && !defined(TARGET_ARM64)
+#if !defined(TARGET_X86) && !defined(TARGET_ARM64) && !defined(TARGET_LOONGARCH64)
     if (stubStartAddress == GetEEFuncEntryPoint(VarargPInvokeStub_RetBuffArg))
         return TRUE;
 #endif
@@ -1841,7 +1839,7 @@ static BOOL IsVarargPInvokeStub(PCODE stubStartAddress)
 BOOL InteropDispatchStubManager::CheckIsStub_Internal(PCODE stubStartAddress)
 {
     WRAPPER_NO_CONTRACT;
-    //@dbgtodo dharvey implement DAC suport
+    //@dbgtodo dharvey implement DAC support
 
 #ifndef DACCESS_COMPILE
 #ifdef FEATURE_COMINTEROP
@@ -1914,13 +1912,13 @@ BOOL InteropDispatchStubManager::TraceManager(Thread *thread,
         _ASSERTE(pNMD->IsNDirect());
         PCODE target = (PCODE)pNMD->GetNDirectTarget();
 
-        LOG((LF_CORDB, LL_INFO10000, "IDSM::TraceManager: Vararg P/Invoke case 0x%x\n", target));
+        LOG((LF_CORDB, LL_INFO10000, "IDSM::TraceManager: Vararg P/Invoke case 0x%p\n", target));
         trace->InitForUnmanaged(target);
     }
     else if (GetIP(pContext) == GetEEFuncEntryPoint(GenericPInvokeCalliHelper))
     {
         PCODE target = (PCODE)arg;
-        LOG((LF_CORDB, LL_INFO10000, "IDSM::TraceManager: Unmanaged CALLI case 0x%x\n", target));
+        LOG((LF_CORDB, LL_INFO10000, "IDSM::TraceManager: Unmanaged CALLI case 0x%p\n", target));
         trace->InitForUnmanaged(target);
     }
 #ifdef FEATURE_COMINTEROP
@@ -1950,7 +1948,7 @@ BOOL InteropDispatchStubManager::TraceManager(Thread *thread,
                 LPVOID *lpVtbl = *(LPVOID **)(IUnknown *)pUnk;
 
                 PCODE target = (PCODE)lpVtbl[6]; // DISPATCH_INVOKE_SLOT;
-                LOG((LF_CORDB, LL_INFO10000, "CPSM::TraceManager: CLR-to-COM late-bound case 0x%x\n", target));
+                LOG((LF_CORDB, LL_INFO10000, "IDSM::TraceManager: CLR-to-COM late-bound case 0x%p\n", target));
                 trace->InitForUnmanaged(target);
 
                 GCPROTECT_END();
@@ -2171,7 +2169,7 @@ BOOL DelegateInvokeStubManager::TraceDelegateObject(BYTE* pbDel, TraceDestinatio
 
     BYTE *pbDelInvocationList = *(BYTE **)(pbDel + DelegateObject::GetOffsetOfInvocationList());
 
-    LOG((LF_CORDB,LL_INFO10000, "DISM::TMI: invocationList: 0x%x\n", pbDelInvocationList));
+    LOG((LF_CORDB,LL_INFO10000, "DISM::TMI: invocationList: 0x%p\n", pbDelInvocationList));
 
     if (pbDelInvocationList == NULL)
     {
@@ -2196,7 +2194,7 @@ BOOL DelegateInvokeStubManager::TraceDelegateObject(BYTE* pbDel, TraceDestinatio
 
         }
 
-        LOG((LF_CORDB,LL_INFO10000, "DISM(DelegateStub)::TM: ppbDest: 0x%x *ppbDest:0x%x\n", ppbDest, *ppbDest));
+        LOG((LF_CORDB,LL_INFO10000, "DISM(DelegateStub)::TM: ppbDest: 0x%p *ppbDest:0x%p\n", ppbDest, *ppbDest));
 
         BOOL res = StubManager::TraceStub((PCODE) (*ppbDest), trace);
 
@@ -2223,7 +2221,7 @@ BOOL DelegateInvokeStubManager::TraceDelegateObject(BYTE* pbDel, TraceDestinatio
             return FALSE;
         }
 
-        LOG((LF_CORDB,LL_INFO10000, "DISM(DelegateStub)::TM: ppbDest: 0x%x *ppbDest:0x%x\n", ppbDest, *ppbDest));
+        LOG((LF_CORDB,LL_INFO10000, "DISM(DelegateStub)::TM: ppbDest: 0x%p *ppbDest:0x%p\n", ppbDest, *ppbDest));
 
         BOOL res = StubManager::TraceStub((PCODE) (*ppbDest), trace);
 
@@ -2382,6 +2380,8 @@ PrecodeStubManager::DoEnumMemoryRegions(CLRDataEnumMemoryFlags flags)
     WRAPPER_NO_CONTRACT;
     DAC_ENUM_VTHIS();
     EMEM_OUT(("MEM: %p PrecodeStubManager\n", dac_cast<TADDR>(this)));
+    GetStubPrecodeRangeList()->EnumMemoryRegions(flags);
+    GetFixupPrecodeRangeList()->EnumMemoryRegions(flags);
 }
 
 void
