@@ -33,10 +33,6 @@ Revision History:
 #define WAIT_ACTIVE         0x02
 #define WAIT_DELETE         0x04
 
-#define TIMER_REGISTERED    0x01
-#define TIMER_ACTIVE        0x02
-#define TIMER_DELETE        0x04
-
 #define WAIT_SINGLE_EXECUTION      0x00000001
 #define WAIT_FREE_CONTEXT          0x00000002
 #define WAIT_INTERNAL_COMPLETION   0x00000004
@@ -97,7 +93,6 @@ class ThreadpoolMgr
     friend class ClrDataAccess;
     friend struct DelegateInfo;
     friend class ThreadPoolNative;
-    friend class TimerNative;
     friend class UnManagedPerAppDomainTPCount;
     friend class ManagedPerAppDomainTPCount;
     friend class PerAppDomainTPCountList;
@@ -147,7 +142,7 @@ public:
             // VolatileLoad may result in torn read
             Counts result;
 #ifndef DACCESS_COMPILE
-            result.AsLongLong = FastInterlockCompareExchangeLong(&counts.AsLongLong, 0, 0);
+            result.AsLongLong = InterlockedCompareExchange64(&counts.AsLongLong, 0, 0);
             ValidateCounts(result);
 #else
             result.AsLongLong = 0; //prevents prefast warning for DAC builds
@@ -180,7 +175,7 @@ public:
             LIMITED_METHOD_CONTRACT;
             Counts result;
 #ifndef DACCESS_COMPILE
-            result.AsLongLong = FastInterlockCompareExchangeLong(&counts.AsLongLong, newCounts.AsLongLong, oldCounts.AsLongLong);
+            result.AsLongLong = InterlockedCompareExchange64(&counts.AsLongLong, newCounts.AsLongLong, oldCounts.AsLongLong);
             if (result == oldCounts)
             {
                 // can only do validation on success; if we failed, it may have been due to a previous
@@ -219,17 +214,21 @@ public:
         MEMTYPE_COUNT           = 3,
     };
 
-    typedef struct {
-        INT32 TimerId;
-    } TimerInfoContext;
-
 #ifndef DACCESS_COMPILE
     static void StaticInitialize()
     {
         WRAPPER_NO_CONTRACT;
+
         s_usePortableThreadPool = CLRConfig::GetConfigValue(CLRConfig::INTERNAL_ThreadPool_UsePortableThreadPool) != 0;
+#ifdef TARGET_WINDOWS
+        s_usePortableThreadPoolForIO =
+            s_usePortableThreadPool &&
+            CLRConfig::GetConfigValue(CLRConfig::INTERNAL_ThreadPool_UsePortableThreadPoolForIO) != 0;
+#else // !TARGET_WINDOWS
+        s_usePortableThreadPoolForIO = s_usePortableThreadPool;
+#endif // TARGET_WINDOWS
     }
-#endif
+#endif // !DACCESS_COMPILE
 
     static bool UsePortableThreadPool()
     {
@@ -237,6 +236,12 @@ public:
         return s_usePortableThreadPool;
     }
 
+    static bool UsePortableThreadPoolForIO()
+    {
+        LIMITED_METHOD_CONTRACT;
+        return s_usePortableThreadPoolForIO;
+    }
+
     static BOOL Initialize();
 
     static BOOL SetMaxThreadsHelper(DWORD MaxWorkerThreads,
@@ -295,13 +300,6 @@ public:
                                             DWORD numBytesTransferred,
                                             LPOVERLAPPED lpOverlapped);
 
-#ifdef TARGET_WINDOWS // the IO completion thread pool is currently only available on Windows
-    static void WINAPI ManagedWaitIOCompletionCallback(DWORD dwErrorCode,
-                                                       DWORD dwNumberOfBytesTransfered,
-                                                       LPOVERLAPPED lpOverlapped);
-#endif
-
-
     static BOOL SetAppDomainRequestsActive(BOOL UnmanagedTP = FALSE);
     static void ClearAppDomainRequestsActive(BOOL UnmanagedTP = FALSE,  LONG index = -1);
 
@@ -311,25 +309,8 @@ public:
         VolatileStore(&LastDequeueTime, (unsigned int)GetTickCount());
     }
 
-    static BOOL CreateTimerQueueTimer(PHANDLE phNewTimer,
-                                        WAITORTIMERCALLBACK Callback,
-                                        PVOID Parameter,
-                                        DWORD DueTime,
-                                        DWORD Period,
-                                        ULONG Flags);
-
-    static BOOL ChangeTimerQueueTimer(HANDLE Timer,
-                                      ULONG DueTime,
-                                      ULONG Period);
-    static BOOL DeleteTimerQueueTimer(HANDLE Timer,
-                                      HANDLE CompletionEvent);
-
     static void RecycleMemory(LPVOID mem, enum MemType memType);
 
-    static void FlushQueueOfTimerInfos();
-
-    static BOOL HaveTimerInfosToFlush() { return TimerInfosToBeRecycled != NULL; }
-
 #ifndef TARGET_UNIX
     static LPOVERLAPPED CompletionPortDispatchWorkWithinAppDomain(Thread* pThread, DWORD* pErrorCode, DWORD* pNumBytes, size_t* pKey);
     static void StoreOverlappedInfoInThread(Thread* pThread, DWORD dwErrorCode, DWORD dwNumBytes, size_t key, LPOVERLAPPED lpOverlapped);
@@ -338,20 +319,10 @@ public:
     // Enable filtering of correlation ETW events for cases handled at a higher abstraction level
 
 #ifndef DACCESS_COMPILE
-    static FORCEINLINE BOOL AreEtwQueueEventsSpeciallyHandled(LPTHREAD_START_ROUTINE Function)
-    {
-        // Timer events are handled at a higher abstraction level: in the managed Timer class
-        return (Function == ThreadpoolMgr::AsyncTimerCallbackCompletion);
-    }
-
     static FORCEINLINE BOOL AreEtwIOQueueEventsSpeciallyHandled(LPOVERLAPPED_COMPLETION_ROUTINE Function)
     {
         // We handle registered waits at a higher abstraction level
-        return (Function == ThreadpoolMgr::WaitIOCompletionCallback
-#ifdef TARGET_WINDOWS // the IO completion thread pool is currently only available on Windows
-                || Function == ThreadpoolMgr::ManagedWaitIOCompletionCallback
-#endif
-            );
+        return Function == ThreadpoolMgr::WaitIOCompletionCallback;
     }
 #endif
 
@@ -565,21 +536,6 @@ private:
         HANDLE      Handle;
     } WaitEvent ;
 
-    // Timer
-    typedef struct {
-        LIST_ENTRY  link;           // doubly linked list of timers
-        ULONG FiringTime;           // TickCount of when to fire next
-        WAITORTIMERCALLBACK Function;             // Function to call when timer fires
-        PVOID Context;              // Context to pass to function when timer fires
-        ULONG Period;
-        DWORD flag;                 // How do we deal with the context
-        DWORD state;
-        LONG refCount;
-        HANDLE ExternalCompletionEvent;     // only one of this is used, but cant do a union since CLREvent has a non-default constructor
-        CLREvent InternalCompletionEvent;   // flags indicates which one is being used
-        OBJECTHANDLE    ExternalEventSafeHandle;
-    } TimerInfo;
-
     static VOID AcquireWaitInfo(WaitInfo *pInfo)
     {
     }
@@ -591,26 +547,8 @@ private:
         pInfo->ExternalCompletionEvent);
 #endif
     }
-    static VOID AcquireTimerInfo(TimerInfo *pInfo)
-    {
-    }
-    static VOID ReleaseTimerInfo(TimerInfo *pInfo)
-    {
-        WRAPPER_NO_CONTRACT;
-#ifndef DACCESS_COMPILE
-        ReleaseInfo(pInfo->ExternalEventSafeHandle,
-        pInfo->ExternalCompletionEvent);
-#endif
-    }
 
     typedef Holder<WaitInfo *, ThreadpoolMgr::AcquireWaitInfo, ThreadpoolMgr::ReleaseWaitInfo> WaitInfoHolder;
-    typedef Holder<TimerInfo *, ThreadpoolMgr::AcquireTimerInfo, ThreadpoolMgr::ReleaseTimerInfo> TimerInfoHolder;
-
-    typedef struct {
-        TimerInfo* Timer;           // timer to be updated
-        ULONG DueTime ;             // new due time
-        ULONG Period ;              // new period
-    } TimerUpdateInfo;
 
     // Definitions and data structures to support recycling of high-frequency
     // memory blocks. We use a spin-lock to access the list
@@ -696,7 +634,7 @@ public:
 
 	        DWORD dwSwitchCount = 0;
 
-	        while(lock != 0 || FastInterlockExchange( &lock, 1 ) != 0)
+	        while(lock != 0 || InterlockedExchange( &lock, 1 ) != 0)
 	        {
                 YieldProcessorNormalized(); // indicate to the processor that we are spinning
 
@@ -752,23 +690,17 @@ public:
 
 #ifndef TARGET_UNIX
             if (CPUGroupInfo::CanEnableThreadUseAllCpuGroups())
-            {
-                // The current processor number may not be within the total number of active processors determined at
-                // initialization time.
-                processorNumber = CPUGroupInfo::CalculateCurrentProcessorNumber() % CPUGroupInfo::GetNumActiveProcessors();
-            }
+                processorNumber = CPUGroupInfo::CalculateCurrentProcessorNumber();
             else
-            {
                 // Turns out GetCurrentProcessorNumber can return a value greater than the number of processors reported by
                 // GetSystemInfo, if we're running in WOW64 on a machine with >32 processors.
-                processorNumber = GetCurrentProcessorNumber() % g_SystemInfo.dwNumberOfProcessors;
-            }
+        	    processorNumber = GetCurrentProcessorNumber()%NumberOfProcessors;
 #else // !TARGET_UNIX
             if (PAL_HasGetCurrentProcessorNumber())
             {
                 // On linux, GetCurrentProcessorNumber which uses sched_getcpu() can return a value greater than the number
                 // of processors reported by sysconf(_SC_NPROCESSORS_ONLN) when using OpenVZ kernel.
-                processorNumber = GetCurrentProcessorNumber() % PAL_GetTotalCpuCount();
+                processorNumber = GetCurrentProcessorNumber()%NumberOfProcessors;
             }
 #endif // !TARGET_UNIX
             return pRecycledListPerProcessor[processorNumber][memType];
@@ -898,7 +830,7 @@ public:
     static BOOL AddWaitRequest(HANDLE waitHandle, WaitInfo* waitInfo);
 
 
-    static ThreadCB* FindWaitThread();              // returns a wait thread that can accomodate another wait request
+    static ThreadCB* FindWaitThread();              // returns a wait thread that can accommodate another wait request
 
     static BOOL CreateWaitThread();
 
@@ -912,16 +844,10 @@ public:
                                 unsigned index,      // array index
                                 BOOL waitTimedOut);
 
-#ifdef TARGET_WINDOWS // the IO completion thread pool is currently only available on Windows
-    static void ManagedWaitIOCompletionCallback_Worker(LPVOID state);
-#endif
-
     static DWORD WINAPI WaitThreadStart(LPVOID lpArgs);
 
     static DWORD WINAPI AsyncCallbackCompletion(PVOID pArgs);
 
-    static void QueueTimerInfoForRelease(TimerInfo *pTimerInfo);
-
     static void DeactivateWait(WaitInfo* waitInfo);
     static void DeactivateNthWait(WaitInfo* waitInfo, DWORD index);
 
@@ -974,15 +900,16 @@ public:
 
 #endif // !TARGET_UNIX
 
+    static void PerformGateActivities(int cpuUtilization);
+    static bool NeedGateThreadForIOCompletions();
+
 private:
     static BOOL IsIoPending();
 
     static BOOL CreateGateThread();
     static void EnsureGateThreadRunning();
-    static bool NeedGateThreadForIOCompletions();
     static bool ShouldGateThreadKeepRunning();
     static DWORD WINAPI GateThreadStart(LPVOID lpArgs);
-    static void PerformGateActivities(int cpuUtilization);
     static BOOL SufficientDelaySinceLastSample(unsigned int LastThreadCreationTime,
                                                unsigned NumThreads, // total number of threads of that type (worker or CP)
                                                double   throttleRate=0.0 // the delay is increased by this percentage for each extra thread
@@ -991,18 +918,6 @@ private:
 
     static LPVOID   GetRecycledMemory(enum MemType memType);
 
-    static DWORD WINAPI TimerThreadStart(LPVOID args);
-    static void TimerThreadFire(); // helper method used by TimerThreadStart
-    static void WINAPI InsertNewTimer(TimerInfo* pArg);
-    static DWORD FireTimers();
-    static DWORD WINAPI AsyncTimerCallbackCompletion(PVOID pArgs);
-    static void DeactivateTimer(TimerInfo* timerInfo);
-    static DWORD WINAPI AsyncDeleteTimer(PVOID pArgs);
-    static void DeleteTimer(TimerInfo* timerInfo);
-    static void WINAPI UpdateTimer(TimerUpdateInfo* pArgs);
-
-    static void WINAPI DeregisterTimer(TimerInfo* pArgs);
-
     inline static DWORD QueueDeregisterWait(HANDLE waitThread, WaitInfo* waitInfo)
     {
         CONTRACTL
@@ -1031,6 +946,7 @@ private:
     static Volatile<LONG> Initialization;                         // indicator of whether the threadpool is initialized.
 
     static bool s_usePortableThreadPool;
+    static bool s_usePortableThreadPoolForIO;
 
     SVAL_DECL(LONG,MinLimitTotalWorkerThreads);         // same as MinLimitTotalCPThreads
     SVAL_DECL(LONG,MaxLimitTotalWorkerThreads);         // same as MaxLimitTotalCPThreads
@@ -1093,13 +1009,6 @@ private:
     static CrstStatic WaitThreadsCriticalSection;
     static LIST_ENTRY WaitThreadsHead;                  // queue of wait threads, each thread can handle upto 64 waits
 
-    static TimerInfo *TimerInfosToBeRecycled;           // list of delegate infos associated with deleted timers
-    static CrstStatic TimerQueueCriticalSection;        // critical section to synchronize timer queue access
-    SVAL_DECL(LIST_ENTRY,TimerQueue);                   // queue of timers
-    static HANDLE TimerThread;                          // Currently we only have one timer thread
-    static Thread*  pTimerThread;
-    DECLSPEC_ALIGN(MAX_CACHE_LINE_SIZE) static DWORD LastTickCount;      // the count just before timer thread goes to sleep
-
     static BOOL InitCompletionPortThreadpool;           // flag indicating whether completion port threadpool has been initialized
     static HANDLE GlobalCompletionPort;                 // used for binding io completions on file handles
 
